searchNodes=[{"type":"module","title":"Matcha","doc":"First-class match specifications for Elixir. Synopsis Matcha offers tight integration with Elixir and match specifications. Match specifications are a BEAM VM feature that can execute de-structuring, pattern matching, and re-structring operations very close-to-the-metal. They can be used to efficiently: filter/map in-memory data find ETS objects trace specific function calls However, they are notoriously difficult to compose and use. Matcha makes this intuitive with ergonomic macros to compose them, and a high-level API with which to use them. Examples # Turn Elixir code into a match specification, # then use it to filter/map some data iex&gt; require Matcha ...&gt; Matcha . spec do ...&gt; { x , y , z } -&gt; x + y + z ...&gt; end ...&gt; |&gt; Matcha.Spec . run! ( [ ...&gt; { 1 , 2 , 3 } , ...&gt; { 1 , 2 } , ...&gt; { 1 , 2 , 3 , 4 } , ...&gt; { 4 , 5 , 6 } ...&gt; ] ) [ 6 , 15 ] This is one way to run test and develop match specifications, but they truly shine in table and tracing applications! Guides Check out the interactive usage guides , including using Matcha for: filtering &amp; mapping data selecting objects from tables tracing function calls","ref":"Matcha.html"},{"type":"macro","title":"Matcha.pattern/1","doc":"Builds a Matcha.Pattern that represents a pattern matching operation on a given input. For more information on match patterns, consult the Matcha.Pattern docs. Examples iex&gt; require Matcha ...&gt; pattern = Matcha . pattern ( { x , y , x } ) # Matcha.Pattern &lt; { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$1&quot; } &gt; iex&gt; Matcha.Pattern . match? ( pattern , { 1 , 2 , 3 } ) false iex&gt; Matcha.Pattern . match? ( pattern , { 1 , 2 , 1 } ) true","ref":"Matcha.html#pattern/1"},{"type":"macro","title":"Matcha.spec/1","doc":"Builds a Matcha.Spec that represents a destructuring, pattern matching, and re-structuring operation on in-memory data. Identical to calling spec/2 with a :filter_map context. Note that this context is mostly used to experiment with match specs, and you should generally prefer calling spec/2 with either a :table or :trace context depending on which Matcha APIs you intend to use: Use the :trace context if you intend to query data with Matcha.Trace functions Use the :table context if you intend to trace code execution with the Matcha.Table functions Examples iex&gt; require Matcha ...&gt; Matcha . spec do ...&gt; { x , y , x } ...&gt; when x &gt; y and y &gt; 0 ...&gt; -&gt; x ...&gt; { x , y , y } ...&gt; when x &lt; y and y &lt; 0 ...&gt; -&gt; y ...&gt; end # Matcha.Spec &lt; [ { { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$1&quot; } , [ { :andalso , { :&gt; , :&quot;$1&quot; , :&quot;$2&quot; } , { :&gt; , :&quot;$2&quot; , 0 } } ] , [ :&quot;$1&quot; ] } , { { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$2&quot; } , [ { :andalso , { :&lt; , :&quot;$1&quot; , :&quot;$2&quot; } , { :&lt; , :&quot;$2&quot; , 0 } } ] , [ :&quot;$2&quot; ] } ] , context : Matcha.Context.FilterMap &gt;","ref":"Matcha.html#spec/1"},{"type":"macro","title":"Matcha.spec/2","doc":"Builds a Matcha.Spec that represents a destructuring, pattern matching, and re-structuring operation in a given context . The context may be :filter_map , :match , :table , :trace , or a Matcha.Context module. This is detailed in the Matcha.Context docs. For more information on match specs, consult the Matcha.Spec docs. Examples iex&gt; require Matcha ...&gt; Matcha . spec ( :table ) do ...&gt; { x , y , x } ...&gt; when x &gt; y and y &gt; 0 ...&gt; -&gt; x ...&gt; { x , y , y } ...&gt; when x &lt; y and y &lt; 0 ...&gt; -&gt; y ...&gt; end # Matcha.Spec &lt; [ { { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$1&quot; } , [ { :andalso , { :&gt; , :&quot;$1&quot; , :&quot;$2&quot; } , { :&gt; , :&quot;$2&quot; , 0 } } ] , [ :&quot;$1&quot; ] } , { { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$2&quot; } , [ { :andalso , { :&lt; , :&quot;$1&quot; , :&quot;$2&quot; } , { :&lt; , :&quot;$2&quot; , 0 } } ] , [ :&quot;$2&quot; ] } ] , context : Matcha.Context.Table &gt;","ref":"Matcha.html#spec/2"},{"type":"macro","title":"Matcha.trace_calls/4","doc":"Traces function calls to module , executing a spec on matching arguments. Tracing is a powerful feature of the BEAM VM, allowing for near zero-cost monitoring of what is happening in running systems. The functions in Matcha.Trace provide utilities for accessing this functionality. One of the most powerful forms of tracing uses match specifications: rather that just print information on when a certain function signature with some number of arguments is invoked, they let you: dissect the arguments in question with pattern-matching and guards take special actions in response (documented in Matcha.Context.Trace ) This macro is a shortcut for constructing a spec with the :trace context via Matcha.spec/2 , and tracing the specified module and function with it via Matcha.Trace.calls/4 . For more information on tracing in general, consult the Matcha.Trace docs. Examples iex&gt; require Matcha ...&gt; Matcha . trace_calls ( Enum , :join , limit : 3 ) do ...&gt; [ _enumerable ] -&gt; message ( &quot;using default joiner&quot; ) ...&gt; [ _enumerable , &quot;&quot; ] -&gt; message ( &quot;using default joiner (but explicitly)&quot; ) ...&gt; [ _enumerable , _custom ] -&gt; message ( &quot;using custom joiner&quot; ) ...&gt; end ...&gt; Enum . join ( 1 .. 3 ) # Prints a trace message with &quot;using default joiner&quot; appended &quot;123&quot; iex&gt; Enum . join ( 1 .. 3 , &quot;&quot; ) # Prints a trace message with &quot;using default joiner (but explicitly)&quot; appended &quot;123&quot; iex&gt; Enum . join ( 1 .. 3 , &quot;, &quot; ) # Prints a trace message with &quot;using custom joiner&quot; appended &quot;1, 2, 3&quot;","ref":"Matcha.html#trace_calls/4"},{"type":"module","title":"Matcha.Pattern","doc":"About patterns.","ref":"Matcha.Pattern.html"},{"type":"function","title":"Matcha.Pattern.filter/2","doc":"","ref":"Matcha.Pattern.html#filter/2"},{"type":"function","title":"Matcha.Pattern.match!/2","doc":"","ref":"Matcha.Pattern.html#match!/2"},{"type":"function","title":"Matcha.Pattern.match?/2","doc":"","ref":"Matcha.Pattern.html#match?/2"},{"type":"function","title":"Matcha.Pattern.source/1","doc":"","ref":"Matcha.Pattern.html#source/1"},{"type":"function","title":"Matcha.Pattern.to_spec/2","doc":"","ref":"Matcha.Pattern.html#to_spec/2"},{"type":"function","title":"Matcha.Pattern.validate/1","doc":"","ref":"Matcha.Pattern.html#validate/1"},{"type":"function","title":"Matcha.Pattern.validate!/1","doc":"","ref":"Matcha.Pattern.html#validate!/1"},{"type":"type","title":"Matcha.Pattern.t/0","doc":"","ref":"Matcha.Pattern.html#t:t/0"},{"type":"module","title":"Matcha.Spec","doc":"About specs.","ref":"Matcha.Spec.html"},{"type":"function","title":"Matcha.Spec.call/2","doc":"","ref":"Matcha.Spec.html#call/2"},{"type":"function","title":"Matcha.Spec.call!/2","doc":"","ref":"Matcha.Spec.html#call!/2"},{"type":"function","title":"Matcha.Spec.from_source/1","doc":"Wraps an existing match specification source code into a Matcha.Spec struct for usage in Matcha APIs. Assumes the spec is written to be used in Matcha.Context.Table context, and validates it as such. To modify this validation behaviour, see from_source/2 . Returns {:ok, %{Matcha.Spec}} if validation succeeds, or {:error, problems} if not.","ref":"Matcha.Spec.html#from_source/1"},{"type":"function","title":"Matcha.Spec.from_source/2","doc":"Wraps an existing match specification source code into a Matcha.Spec struct for usage in Matcha APIs. Accepts a context module or specifier against which to validate. Returns {:ok, %{Matcha.Spec}} if validation succeeds, or {:error, problems} if not.","ref":"Matcha.Spec.html#from_source/2"},{"type":"function","title":"Matcha.Spec.from_source!/1","doc":"Wraps an existing match specification source code into a Matcha.Spec struct for usage in Matcha APIs. Assumes the spec is written to be used in Matcha.Context.Table context, and validates it as such. To modify this validation behaviour, see from_source!/2 . Returns a Matcha.Spec struct if validation succeeds, otherwise raises a Matcha.Spec.Error .","ref":"Matcha.Spec.html#from_source!/1"},{"type":"function","title":"Matcha.Spec.from_source!/2","doc":"Wraps an existing match specification source code into a Matcha.Spec struct for usage in Matcha APIs. Accepts a context module or specifier against which to validate. Returns a Matcha.Spec struct if validation succeeds, otherwise raises a Matcha.Spec.Error .","ref":"Matcha.Spec.html#from_source!/2"},{"type":"function","title":"Matcha.Spec.merge/1","doc":"Merges a list of Matcha.Spec specs into a single matchspec. All specs provided must be built for the same Matcha.Context . The clauses of each matchspec are combined in the order provided into a new matchspec. Note that while the new spec is validated via the validate/1 function at runtime, no compile-time checks are applied (for example, that none of the merged clauses overlap). This means that if an earlier spec provided has a match-all clause; no later clauses can match. This is rarely a problem in practice, as matchspecs tend to not be written with catch-all clauses, since part of their utility is to filter out unwanted matches that are not specified in the spec. Returns {:ok, %{Matcha.Spec}} if the new matchspec is valid, or {:error, problems} if not. Examples iex&gt; require Matcha ...&gt; ...&gt; spec1 = Matcha . spec do ...&gt; integer when is_integer ( integer ) ...&gt; -&gt; integer + 1 ...&gt; end ...&gt; ...&gt; spec2 = Matcha . spec do ...&gt; float when is_float ( float ) ...&gt; -&gt; float + 0.5 ...&gt; end ...&gt; ...&gt; { :ok , merged_spec } = Matcha.Spec . merge ( [ spec1 , spec2 ] ) ...&gt; Matcha.Spec . run! ( merged_spec , [ 1 , 1.5 ] ) [ 2 , 2.0 ]","ref":"Matcha.Spec.html#merge/1"},{"type":"function","title":"Matcha.Spec.merge/2","doc":"Merges spec1 and spec2 into a single matchspec. All specs provided must be built for the same Matcha.Context . See merge/1 for more details on how a merged matchspec behaves. Returns {:ok, %{Matcha.Spec}} if the new matchspec is valid, or {:error, problems} if not.","ref":"Matcha.Spec.html#merge/2"},{"type":"function","title":"Matcha.Spec.merge!/1","doc":"Merges a list of Matcha.Spec specs into a single matchspec. All specs provided must be built for the same Matcha.Context . See merge/1 for more details on how a merged matchspec behaves. Returns the new Matcha.Spec}} if it is valid, or raises a Matcha.Spec exception if not.","ref":"Matcha.Spec.html#merge!/1"},{"type":"function","title":"Matcha.Spec.merge!/2","doc":"Merges spec1 and spec2 into a single matchspec. All specs provided must be built for the same Matcha.Context . See merge/1 for more details on how a merged matchspec behaves. Returns the new Matcha.Spec}} if it is valid, or raises a Matcha.Spec exception if not.","ref":"Matcha.Spec.html#merge!/2"},{"type":"function","title":"Matcha.Spec.run/2","doc":"Runs a match spec over each item in an enumerable . Examples iex&gt; require Matcha ...&gt; Matcha . spec ( :filter_map ) do ...&gt; { amount , tax } when is_integer ( amount ) and amount &gt; 0 -&gt; { :credit , amount + tax } ...&gt; end ...&gt; |&gt; Matcha.Spec . run! ( [ ...&gt; { 9001 , 0 } , ...&gt; { - 200 , - 2.50 } , ...&gt; { - 3 , - 0.5 } , ...&gt; { :error , &quot;bank was offline&quot; } , ...&gt; { 100 , 0 } , ...&gt; { - 743 , - 16.0 } , ...&gt; ] ) [ credit : 9001 , credit : 100 ] Note This function converts the enumerable to a list, which will trigger full enumeration of things like lazy Stream s. If used with an infinite stream, it will run forever! Consider using stream/2 if you need lazy filter/mapping. It isn't as efficient, but plays nicer with infinite streams, and fits into the Stream APIs.","ref":"Matcha.Spec.html#run/2"},{"type":"function","title":"Matcha.Spec.run!/2","doc":"Runs a match spec over each item in an enumerable .","ref":"Matcha.Spec.html#run!/2"},{"type":"function","title":"Matcha.Spec.source/1","doc":"","ref":"Matcha.Spec.html#source/1"},{"type":"function","title":"Matcha.Spec.stream/2","doc":"Produces a Stream that filters out and manipulates elements of an enumerable . Elements of the enumerable that match one of the spec 's clauses will transformed as instructed. Elements that do not match will be filtered out of the result. Always returns a lazy Stream enumerable. Examples # FIXME: context stream logic broken, re-enable after fix # iex&gt; require Matcha ...&gt; Matcha . spec do ...&gt; { amount , tax } when is_integer ( amount ) and amount &lt; 0 -&gt; { :charge , amount + tax } ...&gt; end ...&gt; |&gt; Matcha.Spec . stream ( [ ...&gt; { 9001 , 0 } , ...&gt; { - 200 , - 2.50 } , ...&gt; { - 3 , - 0.5 } , ...&gt; { :error , &quot;bank was offline&quot; } , ...&gt; { 100 , 0 } , ...&gt; { - 743 , - 16.0 } , ...&gt; ] ) ...&gt; |&gt; Stream . take ( 2 ) ...&gt; |&gt; Enum . to_list [ charge : - 202.5 , charge : - 3.5 ] Note This function wraps the enumerable in a lazy Stream . If the enumerable is something you can safely convert to a list without going on forever or loading too much into memory, consider using run/2 instead, as it is more efficient.","ref":"Matcha.Spec.html#stream/2"},{"type":"function","title":"Matcha.Spec.to_pattern/1","doc":"","ref":"Matcha.Spec.html#to_pattern/1"},{"type":"function","title":"Matcha.Spec.valid?/1","doc":"","ref":"Matcha.Spec.html#valid?/1"},{"type":"function","title":"Matcha.Spec.validate/1","doc":"","ref":"Matcha.Spec.html#validate/1"},{"type":"function","title":"Matcha.Spec.validate!/1","doc":"","ref":"Matcha.Spec.html#validate!/1"},{"type":"type","title":"Matcha.Spec.t/0","doc":"","ref":"Matcha.Spec.html#t:t/0"},{"type":"module","title":"Matcha.Table","doc":"","ref":"Matcha.Table.html"},{"type":"macro","title":"Matcha.Table.spec/1","doc":"Builds a Matcha.Spec for table querying purposes. Shorthand for Matcha.spec(:table, spec) .","ref":"Matcha.Table.html#spec/1"},{"type":"module","title":"Matcha.Table.ETS","doc":"","ref":"Matcha.Table.ETS.html"},{"type":"macro","title":"Matcha.Table.ETS.match/3","doc":"","ref":"Matcha.Table.ETS.html#match/3"},{"type":"macro","title":"Matcha.Table.ETS.select/3","doc":"","ref":"Matcha.Table.ETS.html#select/3"},{"type":"type","title":"Matcha.Table.ETS.object/0","doc":"","ref":"Matcha.Table.ETS.html#t:object/0"},{"type":"type","title":"Matcha.Table.ETS.table/0","doc":"","ref":"Matcha.Table.ETS.html#t:table/0"},{"type":"module","title":"Matcha.Table.ETS.Match","doc":"","ref":"Matcha.Table.ETS.Match.html"},{"type":"function","title":"Matcha.Table.ETS.Match.all/2","doc":"","ref":"Matcha.Table.ETS.Match.html#all/2"},{"type":"function","title":"Matcha.Table.ETS.Match.delete/2","doc":"","ref":"Matcha.Table.ETS.Match.html#delete/2"},{"type":"function","title":"Matcha.Table.ETS.Match.object/2","doc":"","ref":"Matcha.Table.ETS.Match.html#object/2"},{"type":"type","title":"Matcha.Table.ETS.Match.operation/0","doc":"","ref":"Matcha.Table.ETS.Match.html#t:operation/0"},{"type":"module","title":"Matcha.Table.ETS.Select","doc":"","ref":"Matcha.Table.ETS.Select.html"},{"type":"function","title":"Matcha.Table.ETS.Select.all/2","doc":"","ref":"Matcha.Table.ETS.Select.html#all/2"},{"type":"function","title":"Matcha.Table.ETS.Select.count/2","doc":"","ref":"Matcha.Table.ETS.Select.html#count/2"},{"type":"function","title":"Matcha.Table.ETS.Select.delete/2","doc":"","ref":"Matcha.Table.ETS.Select.html#delete/2"},{"type":"function","title":"Matcha.Table.ETS.Select.replace/2","doc":"","ref":"Matcha.Table.ETS.Select.html#replace/2"},{"type":"function","title":"Matcha.Table.ETS.Select.reverse/2","doc":"","ref":"Matcha.Table.ETS.Select.html#reverse/2"},{"type":"type","title":"Matcha.Table.ETS.Select.operation/0","doc":"","ref":"Matcha.Table.ETS.Select.html#t:operation/0"},{"type":"module","title":"Matcha.Table.Mnesia","doc":"","ref":"Matcha.Table.Mnesia.html"},{"type":"macro","title":"Matcha.Table.Mnesia.match_object/3","doc":"","ref":"Matcha.Table.Mnesia.html#match_object/3"},{"type":"macro","title":"Matcha.Table.Mnesia.select/3","doc":"","ref":"Matcha.Table.Mnesia.html#select/3"},{"type":"type","title":"Matcha.Table.Mnesia.lock/0","doc":"","ref":"Matcha.Table.Mnesia.html#t:lock/0"},{"type":"type","title":"Matcha.Table.Mnesia.opts/0","doc":"","ref":"Matcha.Table.Mnesia.html#t:opts/0"},{"type":"type","title":"Matcha.Table.Mnesia.table/0","doc":"","ref":"Matcha.Table.Mnesia.html#t:table/0"},{"type":"module","title":"Matcha.Table.Mnesia.Match","doc":"","ref":"Matcha.Table.Mnesia.Match.html"},{"type":"function","title":"Matcha.Table.Mnesia.Match.object/3","doc":"","ref":"Matcha.Table.Mnesia.Match.html#object/3"},{"type":"type","title":"Matcha.Table.Mnesia.Match.operation/0","doc":"","ref":"Matcha.Table.Mnesia.Match.html#t:operation/0"},{"type":"module","title":"Matcha.Table.Mnesia.Select","doc":"","ref":"Matcha.Table.Mnesia.Select.html"},{"type":"function","title":"Matcha.Table.Mnesia.Select.all/3","doc":"","ref":"Matcha.Table.Mnesia.Select.html#all/3"},{"type":"type","title":"Matcha.Table.Mnesia.Select.operation/0","doc":"","ref":"Matcha.Table.Mnesia.Select.html#t:operation/0"},{"type":"module","title":"Matcha.Trace","doc":"About tracing.","ref":"Matcha.Trace.html"},{"type":"function","title":"Matcha.Trace.awaiting_messages?/2","doc":"Checks if pid is awaiting trace messages. Waits timeout milliseconds for the pid to report that all trace messages intended for it when awaiting_messages?/2 was called have been delivered. Returns true if no response is received within timeout , and you may assume that pid is still working through trace messages it has received. If it receives confirmation before the timeout , returns false . The pid must refer to an alive (or previously alive) process from the same node this function is called from , or it will raise an ArgumentError . If the atom :all is provided instead of a pid , this function returns true if any process on the current node is awaiting trace messages. This function is best used when shutting down processes (or the current node), to give them a chance to finish any tracing they are handling.","ref":"Matcha.Trace.html#awaiting_messages?/2"},{"type":"function","title":"Matcha.Trace.calls/4","doc":"Trace function calls to module with specified arguments . arguments may be: an integer arity, only tracing function calls with that number of parameters a Matcha.Spec , only tracing function calls whose arguments match the provided patterns If calling with just an arity, all matching calls will print a corresponding trace message. If calling with a spec, additional operations can be performed, as documented in Matcha.Context.Trace . By default, only 1 calls will be traced. More calls can be traced by providing an integer :limit in the opts . All other opts are forwarded to :recon_trace.calls/3 as the third argument.","ref":"Matcha.Trace.html#calls/4"},{"type":"function","title":"Matcha.Trace.default_formatter/1","doc":"The default formatter for trace messages.","ref":"Matcha.Trace.html#default_formatter/1"},{"type":"function","title":"Matcha.Trace.function/3","doc":"Trace all function calls to module . By default, only 1 calls will be traced. More calls can be traced by providing an integer :limit in the opts . All other opts are forwarded to :recon_trace.calls/3 as the third argument.","ref":"Matcha.Trace.html#function/3"},{"type":"function","title":"Matcha.Trace.info/2","doc":"","ref":"Matcha.Trace.html#info/2"},{"type":"function","title":"Matcha.Trace.module/2","doc":"Trace all calls to a module . By default, only 1 calls will be traced. More calls can be traced by providing an integer :limit in the opts . All other opts are forwarded to :recon_trace.calls/3 as the third argument.","ref":"Matcha.Trace.html#module/2"},{"type":"function","title":"Matcha.Trace.new/1","doc":"","ref":"Matcha.Trace.html#new/1"},{"type":"function","title":"Matcha.Trace.new/2","doc":"","ref":"Matcha.Trace.html#new/2"},{"type":"function","title":"Matcha.Trace.new/3","doc":"","ref":"Matcha.Trace.html#new/3"},{"type":"function","title":"Matcha.Trace.new/4","doc":"Builds a new trace. A custom :formatter function can be provided to opts . It should be a 1-arity function that accepts a trace_info/0 tuple, and returns a message string suitable for consumption by :io.format() .","ref":"Matcha.Trace.html#new/4"},{"type":"macro","title":"Matcha.Trace.spec/1","doc":"Builds a Matcha.Spec for tracing purposes. Shorthand for Matcha.spec(:trace, spec) .","ref":"Matcha.Trace.html#spec/1"},{"type":"function","title":"Matcha.Trace.start/1","doc":"Starts the provided trace .","ref":"Matcha.Trace.html#start/1"},{"type":"function","title":"Matcha.Trace.stop/0","doc":"Stops all tracing at once.","ref":"Matcha.Trace.html#stop/0"},{"type":"type","title":"Matcha.Trace.info_flag/0","doc":"","ref":"Matcha.Trace.html#t:info_flag/0"},{"type":"type","title":"Matcha.Trace.info_item/0","doc":"","ref":"Matcha.Trace.html#t:info_item/0"},{"type":"type","title":"Matcha.Trace.info_item_result/0","doc":"","ref":"Matcha.Trace.html#t:info_item_result/0"},{"type":"type","title":"Matcha.Trace.info_result/0","doc":"","ref":"Matcha.Trace.html#t:info_result/0"},{"type":"type","title":"Matcha.Trace.info_subject/0","doc":"","ref":"Matcha.Trace.html#t:info_subject/0"},{"type":"type","title":"Matcha.Trace.t/0","doc":"","ref":"Matcha.Trace.html#t:t/0"},{"type":"type","title":"Matcha.Trace.trace_info/0","doc":"","ref":"Matcha.Trace.html#t:trace_info/0"},{"type":"type","title":"Matcha.Trace.trace_message/0","doc":"","ref":"Matcha.Trace.html#t:trace_message/0"},{"type":"behaviour","title":"Matcha.Error","doc":"Standard behaviour for Matcha errors.","ref":"Matcha.Error.html"},{"type":"callback","title":"Matcha.Error.format_prelude/1","doc":"Generates the &quot;prelude&quot; text for errors in the struct this error handles into a string displayable in an error message.","ref":"Matcha.Error.html#c:format_prelude/1"},{"type":"function","title":"Matcha.Error.format_problem/1","doc":"","ref":"Matcha.Error.html#format_problem/1"},{"type":"callback","title":"Matcha.Error.format_source/1","doc":"Converts the struct this error handles into a string displayable in an error message.","ref":"Matcha.Error.html#c:format_source/1"},{"type":"type","title":"Matcha.Error.error_problem/0","doc":"","ref":"Matcha.Error.html#t:error_problem/0"},{"type":"type","title":"Matcha.Error.message/0","doc":"","ref":"Matcha.Error.html#t:message/0"},{"type":"type","title":"Matcha.Error.problem/0","doc":"","ref":"Matcha.Error.html#t:problem/0"},{"type":"type","title":"Matcha.Error.problems/0","doc":"","ref":"Matcha.Error.html#t:problems/0"},{"type":"type","title":"Matcha.Error.warning_problem/0","doc":"","ref":"Matcha.Error.html#t:warning_problem/0"},{"type":"exception","title":"Matcha.Pattern.Error","doc":"Error raised when a Matcha.Pattern is invalid.","ref":"Matcha.Pattern.Error.html"},{"type":"function","title":"Matcha.Pattern.Error.message/1","doc":"Produces a human-readable message from the given error .","ref":"Matcha.Pattern.Error.html#message/1"},{"type":"type","title":"Matcha.Pattern.Error.t/0","doc":"","ref":"Matcha.Pattern.Error.html#t:t/0"},{"type":"exception","title":"Matcha.Rewrite.Error","doc":"Error raised when rewriting Elixir code into a match pattern/spec.","ref":"Matcha.Rewrite.Error.html"},{"type":"function","title":"Matcha.Rewrite.Error.message/1","doc":"Produces a human-readable message from the given error .","ref":"Matcha.Rewrite.Error.html#message/1"},{"type":"type","title":"Matcha.Rewrite.Error.t/0","doc":"","ref":"Matcha.Rewrite.Error.html#t:t/0"},{"type":"exception","title":"Matcha.Spec.Error","doc":"Error raised when a Matcha.Spec is invalid.","ref":"Matcha.Spec.Error.html"},{"type":"function","title":"Matcha.Spec.Error.message/1","doc":"Produces a human-readable message from the given error .","ref":"Matcha.Spec.Error.html#message/1"},{"type":"type","title":"Matcha.Spec.Error.t/0","doc":"","ref":"Matcha.Spec.Error.html#t:t/0"},{"type":"exception","title":"Matcha.Trace.Error","doc":"Error raised when trying to trace events happening in a running system.","ref":"Matcha.Trace.Error.html"},{"type":"function","title":"Matcha.Trace.Error.message/1","doc":"Produces a human-readable message from the given error .","ref":"Matcha.Trace.Error.html#message/1"},{"type":"type","title":"Matcha.Trace.Error.t/0","doc":"","ref":"Matcha.Trace.Error.html#t:t/0"},{"type":"behaviour","title":"Matcha.Context","doc":"Different types of match spec are intended to be used for different purposes, and support different instructions in their bodies for different use-cases. The modules implementing the Matcha.Context behaviour define the different types of Matcha.Spec , provide documentation for what specialized instructions that type supports, and are used during Elixir-to-match spec conversion as a concrete function definition to use when expanding instructions (since most of these specialized instructions do not exist anywhere as an actual functions, this lets the Elixir compiler complain about invalid instructions as UndefinedFunctionError s). Predefined contexts Currently there are three applications of match specs supported: :filter_map : Matchspecs intended to be used to filter/map over an in-memory list in an optimized fashion. These types of match spec reference the Matcha.Context.FilterMap module. :match : Matchspecs intended to be used to match over an in-memory list in an optimized fashion. These types of match spec reference the Matcha.Context.Match module. :table : Matchspecs intended to be used to efficiently select data from BEAM VM &quot;table&quot; tools, such as :ets , :dets , and :mnesia , and massage the values returned. These types of match spec reference the Matcha.Context.Table module. :trace : Matchspecs intended to be used to instruct tracing utilities such as :dbg and :recon_trace exactly what function calls with what arguments to trace, and allows invoking special trace command instructions in response. These types of match spec reference the Matcha.Context.Trace module. Custom contexts The context mechanism is technically extensible: any module can implement the Matcha.Context behaviour, define the callbacks, and list public no-op functions to allow their usage in specs compiled with that context (via Matcha.spec(CustomContext) do... ). In practice there is little point in defining a custom context: the supported use-cases for match specs are tightly coupled to the Erlang language, and Matcha covers all of them with its provided contexts, which should be sufficient for any application. The module+behaviour+callback implementation used in Matcha is less about offering extensibility, but instead used to simplify special-casing in Matcha.Spec function implementations, raise Elixir-flavored errors when an invalid instruction is used in the different types of spec, and provide a place to document what they do when invoked.","ref":"Matcha.Context.html"},{"type":"function","title":"Matcha.Context.__core_context_aliases__/0","doc":"Maps the shortcut references to the core Matcha context modules. This describes which shortcuts users may write, for example in Matcha.spec(:some_shortcut) instead of the fully qualified module Matcha.spec(Matcha.Context.SomeContext) .","ref":"Matcha.Context.html#__core_context_aliases__/0"},{"type":"callback","title":"Matcha.Context.__default_match_target__/0","doc":"A default value to use when executing match specs in this context. This function is used to provide Matcha.Source.test/3 with a target value to test against, in situations where it is being used to simply validate the match spec itself, but we do not acutally care if the input matches the spec. This value, when passed to this context's Matcha.Context.__valid_match_target__/1 callback, must produce a true value.","ref":"Matcha.Context.html#c:__default_match_target__/0"},{"type":"callback","title":"Matcha.Context.__emit_erl_test_result__/1","doc":"Decides if the result of a spec match should be part of the result set. This callback runs just after calls to __transform_erl_test_result__/1 or __transform_erl_test_result__/1 . Must return {:emit, result} to include the transformed result of a spec match, when executing it against in-memory data (as opposed to tracing or :ets) for validation or debugging purposes. Otherwise, returning :no_emit will hide the result.","ref":"Matcha.Context.html#c:__emit_erl_test_result__/1"},{"type":"callback","title":"Matcha.Context.__erl_spec_type__/0","doc":"Which primitive Erlang context this context module wraps.","ref":"Matcha.Context.html#c:__erl_spec_type__/0"},{"type":"callback","title":"Matcha.Context.__invalid_match_target_error_message__/1","doc":"Describes an issue with a test target. Invoked to generate a t:Matcha.Error.message when Matcha.Context.__valid_match_target__/1 fails.","ref":"Matcha.Context.html#c:__invalid_match_target_error_message__/1"},{"type":"callback","title":"Matcha.Context.__prepare_source__/1","doc":"Allows this context module to modify match specs before their execution. This hook is the main entrypoint for creating custom contexts, allowing them to augment the match spec with new behaviour when executed in this context. Care must be taken to handle the results of the modified match spec after execution correctly, before they are returned to the caller. This should be implemented in the callbacks: __transform_erl_run_results__/1 __transform_erl_test_result__/1 __emit_erl_test_result__/1","ref":"Matcha.Context.html#c:__prepare_source__/1"},{"type":"callback","title":"Matcha.Context.__transform_erl_run_results__/1","doc":"Transforms the result of a spec match just after calling :ets.match_spec_run/2 . You can think of this as an opportunity to &quot;undo&quot; any modifications to the user's provided matchspec made in __prepare_source__/1 . Must return {:ok, result} to indicate that the returned value is valid; otherwise return {:error, problems} to raise an exception.","ref":"Matcha.Context.html#c:__transform_erl_run_results__/1"},{"type":"callback","title":"Matcha.Context.__transform_erl_test_result__/1","doc":"Transforms the result of a spec match just after calling :erlang.match_spec_test/3 . You can think of this as an opportunity to &quot;undo&quot; any modifications to the user's provided matchspec made in __prepare_source__/1 . Must return {:ok, result} to indicate that the returned value is valid; otherwise return {:error, problems} to raise an exception.","ref":"Matcha.Context.html#c:__transform_erl_test_result__/1"},{"type":"callback","title":"Matcha.Context.__valid_match_target__/1","doc":"A validator that runs before executing a match spec against a target in this context. This validator is run before any match specs are executed on inputs to Matcha.Source.test/3 , and all elements of the enumerable input to Matcha.Source.run/2 . If this function returns false, the match spec will not be executed, instead returning a t:Matcha.Error.error_problem with a t:Matcha.Error.message generated by the Matcha.Context.__invalid_match_target_error_message__/1 callback.","ref":"Matcha.Context.html#c:__valid_match_target__/1"},{"type":"function","title":"Matcha.Context.resolve/1","doc":"Resolves shortcut references to the core Matcha context modules. This allows users to write, for example, Matcha.spec(:trace) instead of the fully qualified module Matcha.spec(Matcha.Context.Trace) .","ref":"Matcha.Context.html#resolve/1"},{"type":"function","title":"Matcha.Context.run/2","doc":"Runs a spec against an enumerable . This is a key function that ensures the input spec and results are passed through the callbacks of a Matcha.Context . Returns either {:ok, results} or {:error, problems} (that other ! APIs may use to raise an exception).","ref":"Matcha.Context.html#run/2"},{"type":"function","title":"Matcha.Context.stream/2","doc":"Creates a lazy Stream that yields the results of running the spec against the provided enumberable . This is a key function that ensures the input spec and results are passed through the callbacks of a Matcha.Context . Returns either {:ok, stream} or {:error, problems} (that other ! APIs may use to raise an exception).","ref":"Matcha.Context.html#stream/2"},{"type":"function","title":"Matcha.Context.supports_compilation?/1","doc":"Determines whether or not specs in this context can be compiled.","ref":"Matcha.Context.html#supports_compilation?/1"},{"type":"function","title":"Matcha.Context.supports_tracing?/1","doc":"Determines whether or not specs in this context can used in tracing.","ref":"Matcha.Context.html#supports_tracing?/1"},{"type":"function","title":"Matcha.Context.test/1","doc":"Tests that the provided spec in its Matcha.Context is valid. Invokes __default_match_target__/0 and passes it into :erlang.match_spec_test/3 . Returns either {:ok, stream} or {:error, problems} (that other ! APIs may use to raise an exception).","ref":"Matcha.Context.html#test/1"},{"type":"function","title":"Matcha.Context.test/2","doc":"Tests that the provided spec in its Matcha.Context correctly matches a provided match_target . Passes the provided match_target into :erlang.match_spec_test/3 . Returns either {:ok, stream} or {:error, problems} (that other ! APIs may use to raise an exception).","ref":"Matcha.Context.html#test/2"},{"type":"type","title":"Matcha.Context.t/0","doc":"","ref":"Matcha.Context.html#t:t/0"},{"type":"module","title":"Matcha.Context.Erlang","doc":"","ref":"Matcha.Context.Erlang.html"},{"type":"function","title":"Matcha.Context.Erlang.*/2","doc":"All match specs can call :erlang.*/2 .","ref":"Matcha.Context.Erlang.html#*/2"},{"type":"function","title":"Matcha.Context.Erlang.+/1","doc":"All match specs can call :erlang.+/1 .","ref":"Matcha.Context.Erlang.html#+/1"},{"type":"function","title":"Matcha.Context.Erlang.+/2","doc":"All match specs can call :erlang.+/2 .","ref":"Matcha.Context.Erlang.html#+/2"},{"type":"function","title":"Matcha.Context.Erlang.-/1","doc":"All match specs can call :erlang.-/1 .","ref":"Matcha.Context.Erlang.html#-/1"},{"type":"function","title":"Matcha.Context.Erlang.-/2","doc":"All match specs can call :erlang.-/2 .","ref":"Matcha.Context.Erlang.html#-/2"},{"type":"function","title":"Matcha.Context.Erlang.//2","doc":"All match specs can call :erlang.//2 .","ref":"Matcha.Context.Erlang.html#//2"},{"type":"function","title":"Matcha.Context.Erlang./=/2","doc":"All match specs can call :erlang./=/2 .","ref":"Matcha.Context.Erlang.html#/=/2"},{"type":"function","title":"Matcha.Context.Erlang.</2","doc":"All match specs can call :erlang.&lt;/2 .","ref":"Matcha.Context.Erlang.html#%3C/2"},{"type":"function","title":"Matcha.Context.Erlang.=/=/2","doc":"All match specs can call :erlang.=/=/2 .","ref":"Matcha.Context.Erlang.html#=/=/2"},{"type":"function","title":"Matcha.Context.Erlang.=:=/2","doc":"All match specs can call :erlang.=:=/2 .","ref":"Matcha.Context.Erlang.html#=:=/2"},{"type":"function","title":"Matcha.Context.Erlang.=</2","doc":"All match specs can call :erlang.=&lt;/2 .","ref":"Matcha.Context.Erlang.html#=%3C/2"},{"type":"function","title":"Matcha.Context.Erlang.==/2","doc":"All match specs can call :erlang.==/2 .","ref":"Matcha.Context.Erlang.html#==/2"},{"type":"function","title":"Matcha.Context.Erlang.>/2","doc":"All match specs can call :erlang.&gt;/2 .","ref":"Matcha.Context.Erlang.html#%3E/2"},{"type":"function","title":"Matcha.Context.Erlang.>=/2","doc":"All match specs can call :erlang.&gt;=/2 .","ref":"Matcha.Context.Erlang.html#%3E=/2"},{"type":"function","title":"Matcha.Context.Erlang.abs/1","doc":"All match specs can call :erlang.abs/1 .","ref":"Matcha.Context.Erlang.html#abs/1"},{"type":"function","title":"Matcha.Context.Erlang.and/2","doc":"All match specs can call :erlang.and/2 .","ref":"Matcha.Context.Erlang.html#and/2"},{"type":"function","title":"Matcha.Context.Erlang.andalso/2","doc":"All match specs can call the andalso/2 short-circuit expression .","ref":"Matcha.Context.Erlang.html#andalso/2"},{"type":"function","title":"Matcha.Context.Erlang.band/2","doc":"All match specs can call :erlang.band/2 .","ref":"Matcha.Context.Erlang.html#band/2"},{"type":"function","title":"Matcha.Context.Erlang.binary_part/2","doc":"All match specs can call :erlang.binary_part/2 .","ref":"Matcha.Context.Erlang.html#binary_part/2"},{"type":"function","title":"Matcha.Context.Erlang.binary_part/3","doc":"All match specs can call :erlang.binary_part/3 .","ref":"Matcha.Context.Erlang.html#binary_part/3"},{"type":"function","title":"Matcha.Context.Erlang.bit_size/1","doc":"All match specs can call :erlang.bit_size/1 .","ref":"Matcha.Context.Erlang.html#bit_size/1"},{"type":"function","title":"Matcha.Context.Erlang.bnot/1","doc":"All match specs can call :erlang.bnot/1 .","ref":"Matcha.Context.Erlang.html#bnot/1"},{"type":"function","title":"Matcha.Context.Erlang.bor/2","doc":"All match specs can call :erlang.bor/2 .","ref":"Matcha.Context.Erlang.html#bor/2"},{"type":"function","title":"Matcha.Context.Erlang.bsl/2","doc":"All match specs can call :erlang.bsl/2 .","ref":"Matcha.Context.Erlang.html#bsl/2"},{"type":"function","title":"Matcha.Context.Erlang.bsr/2","doc":"All match specs can call :erlang.bsr/2 .","ref":"Matcha.Context.Erlang.html#bsr/2"},{"type":"function","title":"Matcha.Context.Erlang.bxor/2","doc":"All match specs can call :erlang.bxor/2 .","ref":"Matcha.Context.Erlang.html#bxor/2"},{"type":"function","title":"Matcha.Context.Erlang.byte_size/1","doc":"All match specs can call :erlang.byte_size/1 .","ref":"Matcha.Context.Erlang.html#byte_size/1"},{"type":"function","title":"Matcha.Context.Erlang.ceil/1","doc":"All match specs can call :erlang.ceil/1 .","ref":"Matcha.Context.Erlang.html#ceil/1"},{"type":"function","title":"Matcha.Context.Erlang.div/2","doc":"All match specs can call :erlang.div/2 .","ref":"Matcha.Context.Erlang.html#div/2"},{"type":"function","title":"Matcha.Context.Erlang.element/2","doc":"All match specs can call :erlang.element/2 .","ref":"Matcha.Context.Erlang.html#element/2"},{"type":"function","title":"Matcha.Context.Erlang.floor/1","doc":"All match specs can call :erlang.floor/1 .","ref":"Matcha.Context.Erlang.html#floor/1"},{"type":"function","title":"Matcha.Context.Erlang.hd/1","doc":"All match specs can call :erlang.hd/1 .","ref":"Matcha.Context.Erlang.html#hd/1"},{"type":"function","title":"Matcha.Context.Erlang.is_atom/1","doc":"All match specs can call :erlang.is_atom/1 .","ref":"Matcha.Context.Erlang.html#is_atom/1"},{"type":"function","title":"Matcha.Context.Erlang.is_binary/1","doc":"All match specs can call :erlang.is_binary/1 .","ref":"Matcha.Context.Erlang.html#is_binary/1"},{"type":"function","title":"Matcha.Context.Erlang.is_float/1","doc":"All match specs can call :erlang.is_float/1 .","ref":"Matcha.Context.Erlang.html#is_float/1"},{"type":"function","title":"Matcha.Context.Erlang.is_function/1","doc":"All match specs can call :erlang.is_function/1 .","ref":"Matcha.Context.Erlang.html#is_function/1"},{"type":"function","title":"Matcha.Context.Erlang.is_function/2","doc":"All match specs can call :erlang.is_function/2 .","ref":"Matcha.Context.Erlang.html#is_function/2"},{"type":"function","title":"Matcha.Context.Erlang.is_integer/1","doc":"All match specs can call :erlang.is_integer/1 .","ref":"Matcha.Context.Erlang.html#is_integer/1"},{"type":"function","title":"Matcha.Context.Erlang.is_list/1","doc":"All match specs can call :erlang.is_list/1 .","ref":"Matcha.Context.Erlang.html#is_list/1"},{"type":"function","title":"Matcha.Context.Erlang.is_map/1","doc":"All match specs can call :erlang.is_map/1 .","ref":"Matcha.Context.Erlang.html#is_map/1"},{"type":"function","title":"Matcha.Context.Erlang.is_map_key/2","doc":"All match specs can call :erlang.is_map_key/2 .","ref":"Matcha.Context.Erlang.html#is_map_key/2"},{"type":"function","title":"Matcha.Context.Erlang.is_number/1","doc":"All match specs can call :erlang.is_number/1 .","ref":"Matcha.Context.Erlang.html#is_number/1"},{"type":"function","title":"Matcha.Context.Erlang.is_pid/1","doc":"All match specs can call :erlang.is_pid/1 .","ref":"Matcha.Context.Erlang.html#is_pid/1"},{"type":"function","title":"Matcha.Context.Erlang.is_port/1","doc":"All match specs can call :erlang.is_port/1 .","ref":"Matcha.Context.Erlang.html#is_port/1"},{"type":"function","title":"Matcha.Context.Erlang.is_record/3","doc":"All match specs can call :erlang.is_record/3 .","ref":"Matcha.Context.Erlang.html#is_record/3"},{"type":"function","title":"Matcha.Context.Erlang.is_reference/1","doc":"All match specs can call :erlang.is_reference/1 .","ref":"Matcha.Context.Erlang.html#is_reference/1"},{"type":"function","title":"Matcha.Context.Erlang.is_tuple/1","doc":"All match specs can call :erlang.is_tuple/1 .","ref":"Matcha.Context.Erlang.html#is_tuple/1"},{"type":"function","title":"Matcha.Context.Erlang.length/1","doc":"All match specs can call :erlang.length/1 .","ref":"Matcha.Context.Erlang.html#length/1"},{"type":"function","title":"Matcha.Context.Erlang.map_get/2","doc":"All match specs can call :erlang.map_get/2 .","ref":"Matcha.Context.Erlang.html#map_get/2"},{"type":"function","title":"Matcha.Context.Erlang.map_size/1","doc":"All match specs can call :erlang.map_size/1 .","ref":"Matcha.Context.Erlang.html#map_size/1"},{"type":"function","title":"Matcha.Context.Erlang.node/0","doc":"All match specs can call :erlang.node/0 .","ref":"Matcha.Context.Erlang.html#node/0"},{"type":"function","title":"Matcha.Context.Erlang.node/1","doc":"All match specs can call :erlang.node/1 .","ref":"Matcha.Context.Erlang.html#node/1"},{"type":"function","title":"Matcha.Context.Erlang.not/1","doc":"All match specs can call :erlang.not/1 .","ref":"Matcha.Context.Erlang.html#not/1"},{"type":"function","title":"Matcha.Context.Erlang.or/2","doc":"All match specs can call :erlang.or/2 .","ref":"Matcha.Context.Erlang.html#or/2"},{"type":"function","title":"Matcha.Context.Erlang.orelse/2","doc":"All match specs can call the orelse/2 short-circuit expression .","ref":"Matcha.Context.Erlang.html#orelse/2"},{"type":"function","title":"Matcha.Context.Erlang.rem/2","doc":"All match specs can call :erlang.rem/2 .","ref":"Matcha.Context.Erlang.html#rem/2"},{"type":"function","title":"Matcha.Context.Erlang.round/1","doc":"All match specs can call :erlang.round/1 .","ref":"Matcha.Context.Erlang.html#round/1"},{"type":"function","title":"Matcha.Context.Erlang.self/0","doc":"All match specs can call :erlang.self/0 .","ref":"Matcha.Context.Erlang.html#self/0"},{"type":"function","title":"Matcha.Context.Erlang.size/1","doc":"All match specs can call :erlang.size/1 .","ref":"Matcha.Context.Erlang.html#size/1"},{"type":"function","title":"Matcha.Context.Erlang.tl/1","doc":"All match specs can call :erlang.tl/1 .","ref":"Matcha.Context.Erlang.html#tl/1"},{"type":"function","title":"Matcha.Context.Erlang.trunc/1","doc":"All match specs can call :erlang.trunc/1 .","ref":"Matcha.Context.Erlang.html#trunc/1"},{"type":"function","title":"Matcha.Context.Erlang.tuple_size/1","doc":"All match specs can call :erlang.tuple_size/1 .","ref":"Matcha.Context.Erlang.html#tuple_size/1"},{"type":"function","title":"Matcha.Context.Erlang.xor/2","doc":"All match specs can call :erlang.xor/2 .","ref":"Matcha.Context.Erlang.html#xor/2"},{"type":"module","title":"Matcha.Context.FilterMap","doc":"Functions and operators that :filter_map match specs can use. ??? Specs created in this context are unique in that they can differentiate between specs that fail to find a matching clause for the given input, and specs with matching clauses that literally return the false value. They return :no_return in the former case, and {:matched, result} tuples in the latter, where result can be a literal false returned from a clause.","ref":"Matcha.Context.FilterMap.html"},{"type":"module","title":"Matcha.Context.Match","doc":"Functions and operators that :match match specs can use. Specs created in this are unique in that they can differentiate between specs that fail to find a matching clause for the given input, and specs with matching clauses that literally return the false value. They return :no_match in the former case, and {:matched, result} tuples in the latter, where result can be a literal false returned from a clause.","ref":"Matcha.Context.Match.html"},{"type":"module","title":"Matcha.Context.Table","doc":"Functions and operators that :table match specs can use. The return values of specs created in this context do not differentiate between specs that fail to find a matching clause for the given input, and specs with matching clauses that literally return the false value; they return {:returned, result} tuples either way.","ref":"Matcha.Context.Table.html"},{"type":"module","title":"Matcha.Context.Trace","doc":"Additional functions that trace match specs can use in their bodies. The return values of specs created in this context do not differentiate between specs that fail to find a matching clause for the given input, and specs with matching clauses that literally return the false value; they return {:traced, result, flags} tuples either way. Tracing match specs offer a wide suite of instructions to drive Erlang's tracing engine in response to matching certain calls. Calls to these functions in match spec bodies will, when that clause is matched, effect the documented change during tracing. These instructions are documented and type-specced here as a convenient reference. For more information, consult the Erlang tracing match spec docs . In addition to general helpful informational functions, tracing supports: Trace Flags Match specs can change how tracing behaves by changing the trace flags on any process. See :erlang.trace/3 for more information. Related functions: enable_trace/1 enabled_trace/2 disable_trace/1 disable_trace/2 trace/2 trace/3 Sequential Tracing Match specs can be used to transfer information between processes via sequential tracing. See the Erlang sequential tracing docs for more information. Related functions: is_seq_trace/0 set_seq_token/2 get_seq_token/0","ref":"Matcha.Context.Trace.html"},{"type":"function","title":"Matcha.Context.Trace.caller/0","doc":"Returns the module/function/arity of the calling function. If the calling function cannot be determined, returns :undefined . This can happen with BIFs in particular.","ref":"Matcha.Context.Trace.html#caller/0"},{"type":"function","title":"Matcha.Context.Trace.disable_trace/1","doc":"Turns off the provided trace_flag for the current process. See the third parameter of :erlang.trace/3 / for a list of flags and their effects. Note that the :cpu_timestamp and :tracer flags are not supported in this function. Always returns true .","ref":"Matcha.Context.Trace.html#disable_trace/1"},{"type":"function","title":"Matcha.Context.Trace.disable_trace/2","doc":"Turns off the provided trace_flag for the specified pid . See the third parameter of :erlang.trace/3 / for a list of flags and their effects. Note that the :cpu_timestamp and :tracer flags are not supported in this function. Always returns true .","ref":"Matcha.Context.Trace.html#disable_trace/2"},{"type":"function","title":"Matcha.Context.Trace.display/1","doc":"Displays the given value on stdout for debugging purposes. Always returns true .","ref":"Matcha.Context.Trace.html#display/1"},{"type":"function","title":"Matcha.Context.Trace.enable_trace/1","doc":"Turns on the provided trace_flag for the current process. See the third parameter of :erlang.trace/3 / for a list of flags and their effects. Note that the :cpu_timestamp and :tracer flags are not supported in this function. Always returns true .","ref":"Matcha.Context.Trace.html#enable_trace/1"},{"type":"function","title":"Matcha.Context.Trace.enable_trace/2","doc":"Turns on the provided trace_flag for the specified pid . See the third parameter of :erlang.trace/3 / for a list of flags and their effects. Note that the :cpu_timestamp and :tracer flags are not supported in this function. Always returns true .","ref":"Matcha.Context.Trace.html#enable_trace/2"},{"type":"function","title":"Matcha.Context.Trace.exception_trace/0","doc":"Works as return_trace/0 , generating an extra exception_from message on exceptions. Causes a return_from trace message to be sent upon return from the current function. Plus, if the traced function exits because of an exception, an exception_from trace message is generated, whether or not the exception is caught . If the process trace flag silent is active, the return_from and exception_from trace messages are inhibited. Always returns true . Warning : If the traced function is tail-recursive, this match specification function destroys that property. Hence, if a match specification executing this function is used on a perpetual server process, t can only be active for a limited period of time, or the emulator will eventually use all memory in the host machine and crash. If this match specification function is inhibited using process trace flag silent, tail-recursiveness still remains.","ref":"Matcha.Context.Trace.html#exception_trace/0"},{"type":"function","title":"Matcha.Context.Trace.get_seq_token/0","doc":"Retreives the (opaque) value of the trace token for the current process. If the current process is not being traced, returns [] . Acts identically to :seq_trace.get_token/0 . The docs say that the return value can be passed back into :seq_trace.set_token/1 . However, in a tracing match spec context, there is no equivalent ( set_seq_token/2 works, but there's no set_seq_token/1 ). So I am unsure what this can be used for. For more information, consult :seq_trace.get_token/0 docs.","ref":"Matcha.Context.Trace.html#get_seq_token/0"},{"type":"function","title":"Matcha.Context.Trace.get_tcw/0","doc":"Returns the value of the current node's trace control word. Identical to calling :erlang.system_info/1 with the argument :trace_control_word . The trace control word is a 32-bit unsigned integer intended for generic trace control. The trace control word can be tested and set both from within trace match specifications and with BIFs.","ref":"Matcha.Context.Trace.html#get_tcw/0"},{"type":"function","title":"Matcha.Context.Trace.is_seq_trace/0","doc":"Returns true if a sequential trace token is set for the current process, otherwise false .","ref":"Matcha.Context.Trace.html#is_seq_trace/0"},{"type":"function","title":"Matcha.Context.Trace.message/1","doc":"Sets an additional message appended to the trace message sent. One can only set one additional message in the body. Later calls replace the appended message. Always returns true . As a special case, {message, false} disables sending of trace messages ('call' and 'return_to') for this function call, just like if the match specification had not matched. This can be useful if only the side effects of the match spec clause's body part are desired. Another special case is {message, true} , which sets the default behavior, as if the function had no match specification; trace message is sent with no extra information (if no other calls to message are placed before {message, true} , it is in fact a &quot;noop&quot;).","ref":"Matcha.Context.Trace.html#message/1"},{"type":"function","title":"Matcha.Context.Trace.process_dump/0","doc":"Returns some textual information about the current process as a binary.","ref":"Matcha.Context.Trace.html#process_dump/0"},{"type":"function","title":"Matcha.Context.Trace.return_trace/0","doc":"Causes a return_from trace message to be sent upon return from the current function. If the process trace flag silent is active, the return_from trace message is inhibited. Always returns true . Warning : If the traced function is tail-recursive, this match specification function destroys that property. Hence, if a match specification executing this function is used on a perpetual server process, it can only be active for a limited period of time, or the emulator will eventually use all memory in the host machine and crash. If this match specification function is inhibited using process trace flag silent, tail-recursiveness still remains.","ref":"Matcha.Context.Trace.html#return_trace/0"},{"type":"function","title":"Matcha.Context.Trace.set_seq_token/2","doc":"Sets a label, serial number, or flag token to value for sequential tracing. Acts like :seq_trace.set_token/2 , except returns true on success, and 'EXIT' on error or bad argument. Note that this function cannot be used to exclude message passing from the trace, since that is normally accomplished by passing [] into :seq_trace.set_token/1 (however there is no set_seq_token/1 allowed in match specs). Note that the values set here cannot be introspected in a match spec tracing context ( get_seq_token/0 returns an opaque representation of the current trace token, but there's no get_seq_token/1 to inspect individual values). For more information, consult :seq_trace.set_token/2 docs.","ref":"Matcha.Context.Trace.html#set_seq_token/2"},{"type":"function","title":"Matcha.Context.Trace.set_tcw/1","doc":"Sets the value of the current node's trace control word to trace_control_word . Identical to calling :erlang.system_flag/2 with the arguments :trace_control_word and trace_control_word . Returns the previous value of the node's trace control word.","ref":"Matcha.Context.Trace.html#set_tcw/1"},{"type":"function","title":"Matcha.Context.Trace.silent/1","doc":"Changes the verbosity of the current process's messaging mode . If mode is true , supresses all trace messages. If mode is false , re-enables trace messages in future calls. If mode is anything else, the current mode remains active.","ref":"Matcha.Context.Trace.html#silent/1"},{"type":"function","title":"Matcha.Context.Trace.trace/2","doc":"Atomically disables and enables a set of trace flags for the current process in one go. Flags enabled in the enable_flags list will override duplicate flags in the disable_flags list. See the third parameter of :erlang.trace/3 / for a list of flags and their effects. Note that the :cpu_timestamp flag is not supported in this function, however unlike the enable_trace/1 and disable_trace/1 functions, the :tracer flags are supported.. If no :tracer is specified, the same tracer as the process executing the match specification is used (not the meta tracer). If that process doesn't have tracer either, then trace flags are ignored. When using a tracer module, the module must be loaded before the match specification is executed. If it is not loaded, the match fails. Returns true if any trace property was changed for the current process, otherwise false .","ref":"Matcha.Context.Trace.html#trace/2"},{"type":"function","title":"Matcha.Context.Trace.trace/3","doc":"Atomically disables and enables a set of trace flags for the given pid in one go. Flags enabled in the enable_flags list will override duplicate flags in the disable_flags list. See the third parameter of :erlang.trace/3 / for a list of flags and their effects. Note that the :cpu_timestamp flag is not supported in this function, however unlike the enable_trace/1 and disable_trace/1 functions, the :tracer flags are supported.. If no :tracer is specified, the same tracer as the process executing the match specification is used (not the meta tracer). If that process doesn't have tracer either, then trace flags are ignored. When using a tracer module, the module must be loaded before the match specification is executed. If it is not loaded, the match fails. Returns true if any trace property was changed for the given pid , otherwise false .","ref":"Matcha.Context.Trace.html#trace/3"},{"type":"type","title":"Matcha.Context.Trace.seq_token/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token/0"},{"type":"type","title":"Matcha.Context.Trace.seq_token_component/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_component/0"},{"type":"type","title":"Matcha.Context.Trace.seq_token_current_serial_number/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_current_serial_number/0"},{"type":"type","title":"Matcha.Context.Trace.seq_token_flag/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_flag/0"},{"type":"type","title":"Matcha.Context.Trace.seq_token_label_value/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_label_value/0"},{"type":"type","title":"Matcha.Context.Trace.seq_token_previous_serial_number/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_previous_serial_number/0"},{"type":"type","title":"Matcha.Context.Trace.seq_token_serial_number/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_serial_number/0"},{"type":"type","title":"Matcha.Context.Trace.seq_token_serial_value/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_serial_value/0"},{"type":"type","title":"Matcha.Context.Trace.seq_token_value/0","doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_value/0"},{"type":"type","title":"Matcha.Context.Trace.trace_flag/0","doc":"","ref":"Matcha.Context.Trace.html#t:trace_flag/0"},{"type":"type","title":"Matcha.Context.Trace.tracer_trace_flag/0","doc":"","ref":"Matcha.Context.Trace.html#t:tracer_trace_flag/0"},{"type":"module","title":"Matcha.Rewrite","doc":"About rewrites.","ref":"Matcha.Rewrite.html"},{"type":"function","title":"Matcha.Rewrite.ast_to_pattern_source/2","doc":"","ref":"Matcha.Rewrite.html#ast_to_pattern_source/2"},{"type":"function","title":"Matcha.Rewrite.ast_to_spec_source/2","doc":"","ref":"Matcha.Rewrite.html#ast_to_spec_source/2"},{"type":"function","title":"Matcha.Rewrite.binding/2","doc":"","ref":"Matcha.Rewrite.html#binding/2"},{"type":"function","title":"Matcha.Rewrite.bound?/2","doc":"","ref":"Matcha.Rewrite.html#bound?/2"},{"type":"macro","title":"Matcha.Rewrite.is_call/1","doc":"","ref":"Matcha.Rewrite.html#is_call/1"},{"type":"macro","title":"Matcha.Rewrite.is_invocation/1","doc":"","ref":"Matcha.Rewrite.html#is_invocation/1"},{"type":"macro","title":"Matcha.Rewrite.is_literal/1","doc":"","ref":"Matcha.Rewrite.html#is_literal/1"},{"type":"macro","title":"Matcha.Rewrite.is_named_var/1","doc":"","ref":"Matcha.Rewrite.html#is_named_var/1"},{"type":"macro","title":"Matcha.Rewrite.is_non_literal/1","doc":"","ref":"Matcha.Rewrite.html#is_non_literal/1"},{"type":"macro","title":"Matcha.Rewrite.is_remote_call/1","doc":"","ref":"Matcha.Rewrite.html#is_remote_call/1"},{"type":"macro","title":"Matcha.Rewrite.is_var/1","doc":"","ref":"Matcha.Rewrite.html#is_var/1"},{"type":"function","title":"Matcha.Rewrite.outer_var?/2","doc":"","ref":"Matcha.Rewrite.html#outer_var?/2"},{"type":"function","title":"Matcha.Rewrite.pattern_to_spec/2","doc":"","ref":"Matcha.Rewrite.html#pattern_to_spec/2"},{"type":"function","title":"Matcha.Rewrite.perform_expansion/2","doc":"","ref":"Matcha.Rewrite.html#perform_expansion/2"},{"type":"function","title":"Matcha.Rewrite.problem/1","doc":"","ref":"Matcha.Rewrite.html#problem/1"},{"type":"function","title":"Matcha.Rewrite.problems/1","doc":"","ref":"Matcha.Rewrite.html#problems/1"},{"type":"function","title":"Matcha.Rewrite.rewrite_bindings/2","doc":"","ref":"Matcha.Rewrite.html#rewrite_bindings/2"},{"type":"function","title":"Matcha.Rewrite.rewrite_body/2","doc":"","ref":"Matcha.Rewrite.html#rewrite_body/2"},{"type":"function","title":"Matcha.Rewrite.rewrite_calls/2","doc":"","ref":"Matcha.Rewrite.html#rewrite_calls/2"},{"type":"function","title":"Matcha.Rewrite.rewrite_conditions/2","doc":"","ref":"Matcha.Rewrite.html#rewrite_conditions/2"},{"type":"function","title":"Matcha.Rewrite.rewrite_match/2","doc":"","ref":"Matcha.Rewrite.html#rewrite_match/2"},{"type":"function","title":"Matcha.Rewrite.source/1","doc":"","ref":"Matcha.Rewrite.html#source/1"},{"type":"function","title":"Matcha.Rewrite.spec_to_pattern/1","doc":"","ref":"Matcha.Rewrite.html#spec_to_pattern/1"},{"type":"function","title":"Matcha.Rewrite.spec_to_pattern!/1","doc":"","ref":"Matcha.Rewrite.html#spec_to_pattern!/1"},{"type":"type","title":"Matcha.Rewrite.ast/0","doc":"","ref":"Matcha.Rewrite.html#t:ast/0"},{"type":"type","title":"Matcha.Rewrite.t/0","doc":"","ref":"Matcha.Rewrite.html#t:t/0"},{"type":"type","title":"Matcha.Rewrite.var_ast/0","doc":"","ref":"Matcha.Rewrite.html#t:var_ast/0"},{"type":"type","title":"Matcha.Rewrite.var_binding/0","doc":"","ref":"Matcha.Rewrite.html#t:var_binding/0"},{"type":"type","title":"Matcha.Rewrite.var_ref/0","doc":"","ref":"Matcha.Rewrite.html#t:var_ref/0"},{"type":"module","title":"Matcha.Rewrite.Kernel","doc":"Replacements for Kernel functions when rewriting Elixir into match specs. These are versions that play nicer with Erlang's match spec limitations.","ref":"Matcha.Rewrite.Kernel.html"},{"type":"macro","title":"Matcha.Rewrite.Kernel.and/2","doc":"Re-implements Kernel.and/2 . This ensures that Elixir 1.6.0+'s boolean optimizations don't create (disallowed) case statements inside match spec bodies.","ref":"Matcha.Rewrite.Kernel.html#and/2"},{"type":"macro","title":"Matcha.Rewrite.Kernel.is_boolean/1","doc":"Re-implements Kernel.is_boolean/1 . The original simply calls out to :erlang.is_boolean/1 , which is not allowed in match specs (as of Erlang/OTP 25). Instead, we re-implement it in terms of things that are. See: https://github.com/erlang/otp/issues/7045","ref":"Matcha.Rewrite.Kernel.html#is_boolean/1"},{"type":"macro","title":"Matcha.Rewrite.Kernel.is_exception/1","doc":"Re-implements Kernel.is_exception/1 . This borrows the guard-specific implementation from Elixir since what Elixir wants to do in normal bodies is invalid in match specs.","ref":"Matcha.Rewrite.Kernel.html#is_exception/1"},{"type":"macro","title":"Matcha.Rewrite.Kernel.is_exception/2","doc":"Re-implements Kernel.is_exception/2 . This borrows the guard-specific implementation from Elixir since what Elixir wants to do in normal bodies is invalid in match specs.","ref":"Matcha.Rewrite.Kernel.html#is_exception/2"},{"type":"macro","title":"Matcha.Rewrite.Kernel.is_struct/1","doc":"Re-implements Kernel.is_struct/1 . This borrows the guard-specific implementation from Elixir since what Elixir wants to do in normal bodies is invalid in match specs.","ref":"Matcha.Rewrite.Kernel.html#is_struct/1"},{"type":"macro","title":"Matcha.Rewrite.Kernel.is_struct/2","doc":"Re-implements Kernel.is_struct/2 . This borrows the guard-specific implementation from Elixir since what Elixir wants to do in normal bodies is invalid in match specs.","ref":"Matcha.Rewrite.Kernel.html#is_struct/2"},{"type":"macro","title":"Matcha.Rewrite.Kernel.or/2","doc":"Re-implements Kernel.or/2 . This ensures that Elixir 1.6.0+'s boolean optimizations don't create (disallowed) case statements inside match spec bodies.","ref":"Matcha.Rewrite.Kernel.html#or/2"},{"type":"module","title":"Matcha.Source","doc":"Functions that work with the raw Erlang terms representing a match spec. The &quot;source&quot; of a match specification is what Matcha calls data that fits the Erlang match specification grammar. Matcha compiles Elixir code into such data, and wraps that data in structs. This module is the bridge between those structs and the Erlang functions that know how to operate on them.","ref":"Matcha.Source.html"},{"type":"function","title":"Matcha.Source.compile/1","doc":"Compiles match spec source into an opaque, more efficient internal representation.","ref":"Matcha.Source.html#compile/1"},{"type":"function","title":"Matcha.Source.compiled?/1","doc":"Checks if provided value is a compiled match spec source.","ref":"Matcha.Source.html#compiled?/1"},{"type":"function","title":"Matcha.Source.ensure_compiled/1","doc":"Ensures provided match spec source is compiled.","ref":"Matcha.Source.html#ensure_compiled/1"},{"type":"function","title":"Matcha.Source.run/2","doc":"Runs a match spec source against a list of values.","ref":"Matcha.Source.html#run/2"},{"type":"function","title":"Matcha.Source.test/3","doc":"Validates match spec source of variant type and tries to match it against match_target .","ref":"Matcha.Source.html#test/3"},{"type":"type","title":"Matcha.Source.all_matches/0","doc":"","ref":"Matcha.Source.html#t:all_matches/0"},{"type":"type","title":"Matcha.Source.body/0","doc":"","ref":"Matcha.Source.html#t:body/0"},{"type":"type","title":"Matcha.Source.clause/0","doc":"","ref":"Matcha.Source.html#t:clause/0"},{"type":"type","title":"Matcha.Source.compiled/0","doc":"","ref":"Matcha.Source.html#t:compiled/0"},{"type":"type","title":"Matcha.Source.condition/0","doc":"","ref":"Matcha.Source.html#t:condition/0"},{"type":"type","title":"Matcha.Source.conditions/0","doc":"","ref":"Matcha.Source.html#t:conditions/0"},{"type":"type","title":"Matcha.Source.expression/0","doc":"","ref":"Matcha.Source.html#t:expression/0"},{"type":"type","title":"Matcha.Source.match_all/0","doc":"","ref":"Matcha.Source.html#t:match_all/0"},{"type":"type","title":"Matcha.Source.match_result/0","doc":"","ref":"Matcha.Source.html#t:match_result/0"},{"type":"type","title":"Matcha.Source.match_target/0","doc":"","ref":"Matcha.Source.html#t:match_target/0"},{"type":"type","title":"Matcha.Source.pattern/0","doc":"","ref":"Matcha.Source.html#t:pattern/0"},{"type":"type","title":"Matcha.Source.spec/0","doc":"","ref":"Matcha.Source.html#t:spec/0"},{"type":"type","title":"Matcha.Source.table_match_result/0","doc":"","ref":"Matcha.Source.html#t:table_match_result/0"},{"type":"type","title":"Matcha.Source.trace_flags/0","doc":"","ref":"Matcha.Source.html#t:trace_flags/0"},{"type":"type","title":"Matcha.Source.trace_match_result/0","doc":"","ref":"Matcha.Source.html#t:trace_match_result/0"},{"type":"type","title":"Matcha.Source.trace_message/0","doc":"","ref":"Matcha.Source.html#t:trace_message/0"},{"type":"type","title":"Matcha.Source.type/0","doc":"","ref":"Matcha.Source.html#t:type/0"},{"type":"type","title":"Matcha.Source.uncompiled/0","doc":"","ref":"Matcha.Source.html#t:uncompiled/0"},{"type":"extras","title":"Using Matcha","doc":"","ref":"guide-usage.html"},{"type":"extras","title":"Using Matcha - Setting up","doc":"The latest version of Matcha can be installed in scripts or iex via: Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } ] , force : true ) IO . puts ( &quot;Installed matcha version: \#{ Application . spec ( :matcha , :vsn ) } &quot; ) The primary entrypoint to Matcha concepts are the macros in the Matcha module, so we require it to make them available: require Matcha Matcha . __info__ ( :macros )","ref":"guide-usage.html#setting-up"},{"type":"extras","title":"Using Matcha - Functionality Overview","doc":"As the macros list suggests, Matcha provides easy ways to: Create match patterns Matcha.Pattern functions can then be used to filter data through them. Create match specifications Matcha.Spec functions can then be used to filter and map data with them. Trace activity in the current runtime Matcha.Trace functions can be used to further interact with the tracing engine. Let's play with each in turn.","ref":"guide-usage.html#functionality-overview"},{"type":"extras","title":"Using Matcha - Match Patterns","doc":"spec = Matcha . spec do { x , y , z } -&gt; { x , y , z } end","ref":"guide-usage.html#match-patterns"},{"type":"extras","title":"...for Filtering/Mapping","doc":"","ref":"guide-filtering-and-mapping.html"},{"type":"extras","title":"...for Filtering/Mapping - Getting started","doc":"Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } ] , force : true ) IO . puts ( &quot;Installed matcha version: \#{ Application . spec ( :matcha , :vsn ) } &quot; ) require Matcha spec = Matcha . spec do { x , y , z } -&gt; { x , y , z } end","ref":"guide-filtering-and-mapping.html#getting-started"},{"type":"extras","title":"...for ETS/DETS/Mnesia","doc":"Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } , { :jason , &quot;&gt;= 0.0.1&quot; } , { :finch , &quot;&gt;= 0.0.1&quot; } ] , force : true ) Finch . start_link ( name : Pokemon.Data ) IO . puts ( &quot;Installed matcha version: \#{ Application . spec ( :matcha , :vsn ) } &quot; ) Resolving Hex dependencies ... Dependency resolution completed : New : castore 0.1 . 19 finch 0.13 . 0 hpax 0.1 . 2 jason 1.4 . 0 mime 2.0 . 3 mint 1.4 . 2 nimble_options 0.4 . 0 nimble_pool 0.2 . 6 recon 2.5 . 2 telemetry 1.1 . 0 * Getting jason ( Hex package ) * Getting finch ( Hex package ) * Getting castore ( Hex package ) * Getting mime ( Hex package ) * Getting mint ( Hex package ) * Getting nimble_options ( Hex package ) * Getting nimble_pool ( Hex package ) * Getting telemetry ( Hex package ) * Getting hpax ( Hex package ) * Getting recon ( Hex package ) == &gt; mime Compiling 1 file ( . ex ) Generated mime app == &gt; nimble_options Compiling 3 files ( . ex ) Generated nimble_options app === &gt; Analyzing applications ... === &gt; Compiling telemetry == &gt; jason Compiling 10 files ( . ex ) Generated jason app == &gt; recon Compiling 6 files ( . erl ) Generated recon app == &gt; hpax Compiling 4 files ( . ex ) Generated hpax app == &gt; nimble_pool Compiling 2 files ( . ex ) Generated nimble_pool app == &gt; matcha Compiling 18 files ( . ex ) Generated matcha app == &gt; castore Compiling 1 file ( . ex ) Generated castore app == &gt; mint Compiling 1 file ( . erl ) Compiling 19 files ( . ex ) Generated mint app == &gt; finch Compiling 13 files ( . ex ) Generated finch app Installed matcha version : 0.1 . 5 :ok","ref":"guide-tables.html"},{"type":"extras","title":"...for ETS/DETS/Mnesia - What Are Tables?","doc":"","ref":"guide-tables.html#what-are-tables"},{"type":"extras","title":"...for ETS/DETS/Mnesia - Seeding Some Data","doc":"Before we can really get cooking using Matcha to extract data from an :ets table, we're going to need to put some data in a table! For fun example datasets, I like to use data about Pokémon . There's a lot of them, and they're fairly well-known — Pokémon being one of the top highest-grossing franchises in human history . We've already created an :ets table called Pokemon.Data in our setup. We've also nabbed the Finch library to make HTTP requests; and Jason to parse JSON datasets into Elixir datastructures. Now we just need a dataset — we'll use @fanzeyi 's data here . { :ok , response } = Finch . build ( :get , &quot;https://raw.githubusercontent.com/fanzeyi/pokemon.json/master/pokedex.json&quot; ) |&gt; Finch . request ( Pokemon.Data ) data = response |&gt; Map . fetch! ( :body ) |&gt; Jason . decode! ( ) [ %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 49 , &quot;Defense&quot; =&gt; 49 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 1 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙种子&quot; , &quot;english&quot; =&gt; &quot;Bulbasaur&quot; , &quot;french&quot; =&gt; &quot;Bulbizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギダネ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 62 , &quot;Defense&quot; =&gt; 63 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 80 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 60 } , &quot;id&quot; =&gt; 2 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙草&quot; , &quot;english&quot; =&gt; &quot;Ivysaur&quot; , &quot;french&quot; =&gt; &quot;Herbizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギソウ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 82 , &quot;Defense&quot; =&gt; 83 , &quot;HP&quot; =&gt; 80 , &quot;Sp. Attack&quot; =&gt; 100 , &quot;Sp. Defense&quot; =&gt; 100 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 3 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙花&quot; , &quot;english&quot; =&gt; &quot;Venusaur&quot; , &quot;french&quot; =&gt; &quot;Florizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギバナ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 52 , &quot;Defense&quot; =&gt; 43 , &quot;HP&quot; =&gt; 39 , &quot;Sp. Attack&quot; =&gt; 60 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 4 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;小火龙&quot; , &quot;english&quot; =&gt; &quot;Charmander&quot; , &quot;french&quot; =&gt; &quot;Salamèche&quot; , &quot;japanese&quot; =&gt; &quot;ヒトカゲ&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 64 , &quot;Defense&quot; =&gt; 58 , &quot;HP&quot; =&gt; 58 , &quot;Sp. Attack&quot; =&gt; 80 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 5 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;火恐龙&quot; , &quot;english&quot; =&gt; &quot;Charmeleon&quot; , &quot;french&quot; =&gt; &quot;Reptincel&quot; , &quot;japanese&quot; =&gt; &quot;リザード&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 84 , &quot;Defense&quot; =&gt; 78 , &quot;HP&quot; =&gt; 78 , &quot;Sp. Attack&quot; =&gt; 109 , &quot;Sp. Defense&quot; =&gt; 85 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 6 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;喷火龙&quot; , &quot;english&quot; =&gt; &quot;Charizard&quot; , &quot;french&quot; =&gt; &quot;Dracaufeu&quot; , &quot;japanese&quot; =&gt; &quot;リザードン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 48 , &quot;Defense&quot; =&gt; 65 , &quot;HP&quot; =&gt; 44 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 64 , &quot;Speed&quot; =&gt; 43 } , &quot;id&quot; =&gt; 7 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;杰尼龟&quot; , &quot;english&quot; =&gt; &quot;Squirtle&quot; , &quot;french&quot; =&gt; &quot;Carapuce&quot; , &quot;japanese&quot; =&gt; &quot;ゼニガメ&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 63 , &quot;Defense&quot; =&gt; 80 , &quot;HP&quot; =&gt; 59 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 58 } , &quot;id&quot; =&gt; 8 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;卡咪龟&quot; , &quot;english&quot; =&gt; &quot;Wartortle&quot; , &quot;french&quot; =&gt; &quot;Carabaffe&quot; , &quot;japanese&quot; =&gt; &quot;カメール&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 83 , &quot;Defense&quot; =&gt; 100 , &quot;HP&quot; =&gt; 79 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 105 , &quot;Speed&quot; =&gt; 78 } , &quot;id&quot; =&gt; 9 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;水箭龟&quot; , &quot;english&quot; =&gt; &quot;Blastoise&quot; , &quot;french&quot; =&gt; &quot;Tortank&quot; , &quot;japanese&quot; =&gt; &quot;カメックス&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 30 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 20 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 10 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;绿毛虫&quot; , &quot;english&quot; =&gt; &quot;Caterpie&quot; , &quot;french&quot; =&gt; &quot;Chenipan&quot; , &quot;japanese&quot; =&gt; &quot;キャタピー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 20 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 50 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 30 } , &quot;id&quot; =&gt; 11 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;铁甲蛹&quot; , &quot;english&quot; =&gt; &quot;Metapod&quot; , &quot;french&quot; =&gt; &quot;Chrysacier&quot; , &quot;japanese&quot; =&gt; &quot;トランセル&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 50 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 90 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 70 } , &quot;id&quot; =&gt; 12 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;巴大蝶&quot; , &quot;english&quot; =&gt; &quot;Butterfree&quot; , &quot;french&quot; =&gt; &quot;Papilusion&quot; , &quot;japanese&quot; =&gt; &quot;バタフリー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 35 , &quot;Defense&quot; =&gt; 30 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 20 , &quot;Speed&quot; =&gt; 50 } , &quot;id&quot; =&gt; 13 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;独角虫&quot; , &quot;english&quot; =&gt; &quot;Weedle&quot; , &quot;french&quot; =&gt; &quot;Aspicot&quot; , &quot;japanese&quot; =&gt; &quot;ビードル&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 25 , &quot;Defense&quot; =&gt; 50 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 35 } , &quot;id&quot; =&gt; 14 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;铁壳蛹&quot; , &quot;english&quot; =&gt; &quot;Kakuna&quot; , &quot;french&quot; =&gt; &quot;Coconfort&quot; , &quot;japanese&quot; =&gt; &quot;コクーン&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 65 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 75 } , &quot;id&quot; =&gt; 15 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大针蜂&quot; , &quot;english&quot; =&gt; &quot;Beedrill&quot; , &quot;french&quot; =&gt; &quot;Dardargnan&quot; , &quot;japanese&quot; =&gt; &quot;スピアー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 35 , &quot;Sp. Defense&quot; =&gt; 35 , &quot;Speed&quot; =&gt; 56 } , &quot;id&quot; =&gt; 16 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;波波&quot; , &quot;english&quot; =&gt; &quot;Pidgey&quot; , &quot;french&quot; =&gt; &quot;Roucool&quot; , &quot;japanese&quot; =&gt; &quot;ポッポ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 63 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 71 } , &quot;id&quot; =&gt; 17 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;比比鸟&quot; , &quot;english&quot; =&gt; &quot;Pidgeotto&quot; , &quot;french&quot; =&gt; &quot;Roucoups&quot; , &quot;japanese&quot; =&gt; &quot;ピジョン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 75 , &quot;HP&quot; =&gt; 83 , &quot;Sp. Attack&quot; =&gt; 70 , &quot;Sp. Defense&quot; =&gt; 70 , &quot;Speed&quot; =&gt; 101 } , &quot;id&quot; =&gt; 18 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大比鸟&quot; , &quot;english&quot; =&gt; &quot;Pidgeot&quot; , &quot;french&quot; =&gt; &quot;Roucarnage&quot; , &quot;japanese&quot; =&gt; &quot;ピジョット&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 56 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 30 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 35 , &quot;Speed&quot; =&gt; 72 } , &quot;id&quot; =&gt; 19 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;小拉达&quot; , &quot;english&quot; =&gt; &quot;Rattata&quot; , &quot;french&quot; =&gt; &quot;Rattata&quot; , &quot;japanese&quot; =&gt; &quot;コラッタ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 81 , &quot;Defense&quot; =&gt; 60 , &quot;HP&quot; =&gt; 55 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 70 , &quot;Speed&quot; =&gt; 97 } , &quot;id&quot; =&gt; 20 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;拉达&quot; , &quot;english&quot; =&gt; &quot;Raticate&quot; , &quot;french&quot; =&gt; &quot;Rattatac&quot; , &quot;japanese&quot; =&gt; &quot;ラッタ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 30 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 31 , &quot;Sp. Defense&quot; =&gt; 31 , &quot;Speed&quot; =&gt; 70 } , &quot;id&quot; =&gt; 21 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;烈雀&quot; , &quot;english&quot; =&gt; &quot;Spearow&quot; , &quot;french&quot; =&gt; &quot;Piafabec&quot; , &quot;japanese&quot; =&gt; &quot;オニスズメ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 65 , &quot;HP&quot; =&gt; 65 , &quot;Sp. Attack&quot; =&gt; 61 , &quot;Sp. Defense&quot; =&gt; 61 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 22 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大嘴雀&quot; , &quot;english&quot; =&gt; &quot;Fearow&quot; , &quot;french&quot; =&gt; &quot;Rapasdepic&quot; , &quot;japanese&quot; =&gt; &quot;オニドリル&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 44 , &quot;HP&quot; =&gt; 35 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 54 , &quot;Speed&quot; =&gt; 55 } , &quot;id&quot; =&gt; 23 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;阿柏蛇&quot; , &quot;english&quot; =&gt; &quot;Ekans&quot; , &quot;french&quot; =&gt; &quot;Abo&quot; , &quot;japanese&quot; =&gt; &quot;アーボ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 95 , &quot;Defense&quot; =&gt; 69 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 79 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 24 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;阿柏怪&quot; , &quot;english&quot; =&gt; &quot;Arbok&quot; , &quot;french&quot; =&gt; &quot;Arbok&quot; , &quot;japanese&quot; =&gt; &quot;アーボック&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 55 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 35 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 90 } , &quot;id&quot; =&gt; 25 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮卡丘&quot; , &quot;english&quot; =&gt; &quot;Pikachu&quot; , &quot;french&quot; =&gt; &quot;Pikachu&quot; , &quot;japanese&quot; =&gt; &quot;ピカチュウ&quot; } , &quot;type&quot; =&gt; [ &quot;Electric&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 90 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 110 } , &quot;id&quot; =&gt; 26 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;雷丘&quot; , &quot;english&quot; =&gt; &quot;Raichu&quot; , &quot;french&quot; =&gt; &quot;Raichu&quot; , &quot;japanese&quot; =&gt; &quot;ライチュウ&quot; } , &quot;type&quot; =&gt; [ &quot;Electric&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 75 , &quot;Defense&quot; =&gt; 85 , &quot;HP&quot; =&gt; 50 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 30 , &quot;Speed&quot; =&gt; 40 } , &quot;id&quot; =&gt; 27 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;穿山鼠&quot; , &quot;english&quot; =&gt; &quot;Sandshrew&quot; , &quot;french&quot; =&gt; &quot;Sabelette&quot; , &quot;japanese&quot; =&gt; &quot;サンド&quot; } , &quot;type&quot; =&gt; [ &quot;Ground&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 100 , &quot;Defense&quot; =&gt; 110 , &quot;HP&quot; =&gt; 75 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 28 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;穿山王&quot; , &quot;english&quot; =&gt; &quot;Sandslash&quot; , &quot;french&quot; =&gt; &quot;Sablaireau&quot; , &quot;japanese&quot; =&gt; &quot;サンドパン&quot; } , &quot;type&quot; =&gt; [ &quot;Ground&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 47 , &quot;Defense&quot; =&gt; 52 , &quot;HP&quot; =&gt; 55 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 41 } , &quot;id&quot; =&gt; 29 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多兰&quot; , &quot;english&quot; =&gt; &quot;Nidoran♀&quot; , &quot;french&quot; =&gt; &quot;Nidoran♀&quot; , &quot;japanese&quot; =&gt; &quot;ニドラン♀&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 62 , &quot;Defense&quot; =&gt; 67 , &quot;HP&quot; =&gt; 70 , &quot;Sp. Attack&quot; =&gt; 55 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 56 } , &quot;id&quot; =&gt; 30 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多娜&quot; , &quot;english&quot; =&gt; &quot;Nidorina&quot; , &quot;french&quot; =&gt; &quot;Nidorina&quot; , &quot;japanese&quot; =&gt; &quot;ニドリーナ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 92 , &quot;Defense&quot; =&gt; 87 , &quot;HP&quot; =&gt; 90 , &quot;Sp. Attack&quot; =&gt; 75 , &quot;Sp. Defense&quot; =&gt; 85 , &quot;Speed&quot; =&gt; 76 } , &quot;id&quot; =&gt; 31 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多后&quot; , &quot;english&quot; =&gt; &quot;Nidoqueen&quot; , &quot;french&quot; =&gt; &quot;Nidoqueen&quot; , &quot;japanese&quot; =&gt; &quot;ニドクイン&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Ground&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 57 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 46 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 50 } , &quot;id&quot; =&gt; 32 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多朗&quot; , &quot;english&quot; =&gt; &quot;Nidoran♂&quot; , &quot;french&quot; =&gt; &quot;Nidoran♂&quot; , &quot;japanese&quot; =&gt; &quot;ニドラン♂&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 72 , &quot;Defense&quot; =&gt; 57 , &quot;HP&quot; =&gt; 61 , &quot;Sp. Attack&quot; =&gt; 55 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 33 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多力诺&quot; , &quot;english&quot; =&gt; &quot;Nidorino&quot; , &quot;french&quot; =&gt; &quot;Nidorino&quot; , &quot;japanese&quot; =&gt; &quot;ニドリーノ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 102 , &quot;Defense&quot; =&gt; 77 , &quot;HP&quot; =&gt; 81 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 75 , &quot;Speed&quot; =&gt; 85 } , &quot;id&quot; =&gt; 34 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多王&quot; , &quot;english&quot; =&gt; &quot;Nidoking&quot; , &quot;french&quot; =&gt; &quot;Nidoking&quot; , &quot;japanese&quot; =&gt; &quot;ニドキング&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Ground&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 48 , &quot;HP&quot; =&gt; 70 , &quot;Sp. Attack&quot; =&gt; 60 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 35 } , &quot;id&quot; =&gt; 35 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮皮&quot; , &quot;english&quot; =&gt; &quot;Clefairy&quot; , &quot;french&quot; =&gt; &quot;Mélofée&quot; , &quot;japanese&quot; =&gt; &quot;ピッピ&quot; } , &quot;type&quot; =&gt; [ &quot;Fairy&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , &quot;Defense&quot; =&gt; 73 , &quot;HP&quot; =&gt; 95 , &quot;Sp. Attack&quot; =&gt; 95 , &quot;Sp. Defense&quot; =&gt; 90 , &quot;Speed&quot; =&gt; 60 } , &quot;id&quot; =&gt; 36 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮可西&quot; , &quot;english&quot; =&gt; &quot;Clefable&quot; , &quot;french&quot; =&gt; &quot;Mélodelfe&quot; , &quot;japanese&quot; =&gt; &quot;ピクシー&quot; } , &quot;type&quot; =&gt; [ &quot;Fairy&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 41 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 38 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 37 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;六尾&quot; , &quot;english&quot; =&gt; &quot;Vulpix&quot; , &quot;french&quot; =&gt; &quot;Goupix&quot; , &quot;japanese&quot; =&gt; &quot;ロコン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 76 , &quot;Defense&quot; =&gt; 75 , &quot;HP&quot; =&gt; 73 , &quot;Sp. Attack&quot; =&gt; 81 , &quot;Sp. Defense&quot; =&gt; 100 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 38 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;九尾&quot; , &quot;english&quot; =&gt; &quot;Ninetales&quot; , &quot;french&quot; =&gt; &quot;Feunard&quot; , &quot;japanese&quot; =&gt; &quot;キュウコン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 20 , &quot;HP&quot; =&gt; 115 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 20 } , &quot;id&quot; =&gt; 39 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;胖丁&quot; , &quot;english&quot; =&gt; &quot;Jigglypuff&quot; , &quot;french&quot; =&gt; &quot;Rondoudou&quot; , &quot;japanese&quot; =&gt; &quot;プリン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Fairy&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , &quot;Defense&quot; =&gt; 45 , &quot;HP&quot; =&gt; 140 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 40 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;胖可丁&quot; , &quot;english&quot; =&gt; &quot;Wigglytuff&quot; , &quot;french&quot; =&gt; &quot;Grodoudou&quot; , &quot;japanese&quot; =&gt; &quot;プクリン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Fairy&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 30 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 55 } , &quot;id&quot; =&gt; 41 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;超音蝠&quot; , &quot;english&quot; =&gt; &quot;Zubat&quot; , &quot;french&quot; =&gt; &quot;Nosferapti&quot; , &quot;japanese&quot; =&gt; &quot;ズバット&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 70 , &quot;HP&quot; =&gt; 75 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 75 , &quot;Speed&quot; =&gt; 90 } , &quot;id&quot; =&gt; 42 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大嘴蝠&quot; , &quot;english&quot; =&gt; &quot;Golbat&quot; , &quot;french&quot; =&gt; &quot;Nosferalto&quot; , &quot;japanese&quot; =&gt; &quot;ゴルバット&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 50 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 75 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 30 } , &quot;id&quot; =&gt; 43 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;走路草&quot; , &quot;english&quot; =&gt; &quot;Oddish&quot; , &quot;french&quot; =&gt; &quot;Mystherbe&quot; , &quot;japanese&quot; =&gt; &quot;ナゾノクサ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 65 , &quot;Defense&quot; =&gt; 70 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 75 , ... } , &quot;id&quot; =&gt; 44 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;臭臭花&quot; , &quot;english&quot; =&gt; &quot;Gloom&quot; , &quot;french&quot; =&gt; &quot;Ortide&quot; , ... } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 85 , &quot;HP&quot; =&gt; 75 , &quot;Sp. Attack&quot; =&gt; 110 , ... } , &quot;id&quot; =&gt; 45 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;霸王花&quot; , &quot;english&quot; =&gt; &quot;Vileplume&quot; , ... } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , ... ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 35 , ... } , &quot;id&quot; =&gt; 46 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;派拉斯&quot; , ... } , &quot;type&quot; =&gt; [ ... ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 95 , &quot;Defense&quot; =&gt; 80 , ... } , &quot;id&quot; =&gt; 47 , &quot;name&quot; =&gt; %{ ... } , ... } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 55 , ... } , &quot;id&quot; =&gt; 48 , ... } , %{ &quot;base&quot; =&gt; %{ ... } , ... } , %{ ... } , ... ] :ets objects take the form of an n-tuple, with (usually) the first item in the tuple being the &quot;key&quot; we recognize the object by. We'll massage our dataset to fit this structure: in our case, {id, pokemon_data} , extracting each pokemon's numeric &quot;id&quot; as our key from the pokemon_data to index it by: pokedex_objects = data |&gt; Enum . map ( fn pokemon_data -&gt; { Map . fetch! ( pokemon_data , &quot;id&quot; ) , pokemon_data } end ) |&gt; Enum . sort_by ( &amp; elem ( &amp;1 , 0 ) ) [ { 1 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 49 , &quot;Defense&quot; =&gt; 49 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 1 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙种子&quot; , &quot;english&quot; =&gt; &quot;Bulbasaur&quot; , &quot;french&quot; =&gt; &quot;Bulbizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギダネ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } } , { 2 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 62 , &quot;Defense&quot; =&gt; 63 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 80 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 60 } , &quot;id&quot; =&gt; 2 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙草&quot; , &quot;english&quot; =&gt; &quot;Ivysaur&quot; , &quot;french&quot; =&gt; &quot;Herbizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギソウ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } } , { 3 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 82 , &quot;Defense&quot; =&gt; 83 , &quot;HP&quot; =&gt; 80 , &quot;Sp. Attack&quot; =&gt; 100 , &quot;Sp. Defense&quot; =&gt; 100 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 3 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙花&quot; , &quot;english&quot; =&gt; &quot;Venusaur&quot; , &quot;french&quot; =&gt; &quot;Florizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギバナ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } } , { 4 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 52 , &quot;Defense&quot; =&gt; 43 , &quot;HP&quot; =&gt; 39 , &quot;Sp. Attack&quot; =&gt; 60 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 4 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;小火龙&quot; , &quot;english&quot; =&gt; &quot;Charmander&quot; , &quot;french&quot; =&gt; &quot;Salamèche&quot; , &quot;japanese&quot; =&gt; &quot;ヒトカゲ&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } } , { 5 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 64 , &quot;Defense&quot; =&gt; 58 , &quot;HP&quot; =&gt; 58 , &quot;Sp. Attack&quot; =&gt; 80 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 5 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;火恐龙&quot; , &quot;english&quot; =&gt; &quot;Charmeleon&quot; , &quot;french&quot; =&gt; &quot;Reptincel&quot; , &quot;japanese&quot; =&gt; &quot;リザード&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } } , { 6 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 84 , &quot;Defense&quot; =&gt; 78 , &quot;HP&quot; =&gt; 78 , &quot;Sp. Attack&quot; =&gt; 109 , &quot;Sp. Defense&quot; =&gt; 85 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 6 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;喷火龙&quot; , &quot;english&quot; =&gt; &quot;Charizard&quot; , &quot;french&quot; =&gt; &quot;Dracaufeu&quot; , &quot;japanese&quot; =&gt; &quot;リザードン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; , &quot;Flying&quot; ] } } , { 7 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 48 , &quot;Defense&quot; =&gt; 65 , &quot;HP&quot; =&gt; 44 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 64 , &quot;Speed&quot; =&gt; 43 } , &quot;id&quot; =&gt; 7 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;杰尼龟&quot; , &quot;english&quot; =&gt; &quot;Squirtle&quot; , &quot;french&quot; =&gt; &quot;Carapuce&quot; , &quot;japanese&quot; =&gt; &quot;ゼニガメ&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } } , { 8 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 63 , &quot;Defense&quot; =&gt; 80 , &quot;HP&quot; =&gt; 59 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 58 } , &quot;id&quot; =&gt; 8 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;卡咪龟&quot; , &quot;english&quot; =&gt; &quot;Wartortle&quot; , &quot;french&quot; =&gt; &quot;Carabaffe&quot; , &quot;japanese&quot; =&gt; &quot;カメール&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } } , { 9 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 83 , &quot;Defense&quot; =&gt; 100 , &quot;HP&quot; =&gt; 79 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 105 , &quot;Speed&quot; =&gt; 78 } , &quot;id&quot; =&gt; 9 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;水箭龟&quot; , &quot;english&quot; =&gt; &quot;Blastoise&quot; , &quot;french&quot; =&gt; &quot;Tortank&quot; , &quot;japanese&quot; =&gt; &quot;カメックス&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } } , { 10 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 30 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 20 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 10 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;绿毛虫&quot; , &quot;english&quot; =&gt; &quot;Caterpie&quot; , &quot;french&quot; =&gt; &quot;Chenipan&quot; , &quot;japanese&quot; =&gt; &quot;キャタピー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; ] } } , { 11 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 20 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 50 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 30 } , &quot;id&quot; =&gt; 11 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;铁甲蛹&quot; , &quot;english&quot; =&gt; &quot;Metapod&quot; , &quot;french&quot; =&gt; &quot;Chrysacier&quot; , &quot;japanese&quot; =&gt; &quot;トランセル&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; ] } } , { 12 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 50 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 90 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 70 } , &quot;id&quot; =&gt; 12 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;巴大蝶&quot; , &quot;english&quot; =&gt; &quot;Butterfree&quot; , &quot;french&quot; =&gt; &quot;Papilusion&quot; , &quot;japanese&quot; =&gt; &quot;バタフリー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Flying&quot; ] } } , { 13 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 35 , &quot;Defense&quot; =&gt; 30 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 20 , &quot;Speed&quot; =&gt; 50 } , &quot;id&quot; =&gt; 13 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;独角虫&quot; , &quot;english&quot; =&gt; &quot;Weedle&quot; , &quot;french&quot; =&gt; &quot;Aspicot&quot; , &quot;japanese&quot; =&gt; &quot;ビードル&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } } , { 14 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 25 , &quot;Defense&quot; =&gt; 50 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 35 } , &quot;id&quot; =&gt; 14 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;铁壳蛹&quot; , &quot;english&quot; =&gt; &quot;Kakuna&quot; , &quot;french&quot; =&gt; &quot;Coconfort&quot; , &quot;japanese&quot; =&gt; &quot;コクーン&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } } , { 15 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 65 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 75 } , &quot;id&quot; =&gt; 15 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大针蜂&quot; , &quot;english&quot; =&gt; &quot;Beedrill&quot; , &quot;french&quot; =&gt; &quot;Dardargnan&quot; , &quot;japanese&quot; =&gt; &quot;スピアー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } } , { 16 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 35 , &quot;Sp. Defense&quot; =&gt; 35 , &quot;Speed&quot; =&gt; 56 } , &quot;id&quot; =&gt; 16 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;波波&quot; , &quot;english&quot; =&gt; &quot;Pidgey&quot; , &quot;french&quot; =&gt; &quot;Roucool&quot; , &quot;japanese&quot; =&gt; &quot;ポッポ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } } , { 17 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 63 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 71 } , &quot;id&quot; =&gt; 17 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;比比鸟&quot; , &quot;english&quot; =&gt; &quot;Pidgeotto&quot; , &quot;french&quot; =&gt; &quot;Roucoups&quot; , &quot;japanese&quot; =&gt; &quot;ピジョン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } } , { 18 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 75 , &quot;HP&quot; =&gt; 83 , &quot;Sp. Attack&quot; =&gt; 70 , &quot;Sp. Defense&quot; =&gt; 70 , &quot;Speed&quot; =&gt; 101 } , &quot;id&quot; =&gt; 18 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大比鸟&quot; , &quot;english&quot; =&gt; &quot;Pidgeot&quot; , &quot;french&quot; =&gt; &quot;Roucarnage&quot; , &quot;japanese&quot; =&gt; &quot;ピジョット&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } } , { 19 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 56 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 30 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 35 , &quot;Speed&quot; =&gt; 72 } , &quot;id&quot; =&gt; 19 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;小拉达&quot; , &quot;english&quot; =&gt; &quot;Rattata&quot; , &quot;french&quot; =&gt; &quot;Rattata&quot; , &quot;japanese&quot; =&gt; &quot;コラッタ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; ] } } , { 20 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 81 , &quot;Defense&quot; =&gt; 60 , &quot;HP&quot; =&gt; 55 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 70 , &quot;Speed&quot; =&gt; 97 } , &quot;id&quot; =&gt; 20 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;拉达&quot; , &quot;english&quot; =&gt; &quot;Raticate&quot; , &quot;french&quot; =&gt; &quot;Rattatac&quot; , &quot;japanese&quot; =&gt; &quot;ラッタ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; ] } } , { 21 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 30 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 31 , &quot;Sp. Defense&quot; =&gt; 31 , &quot;Speed&quot; =&gt; 70 } , &quot;id&quot; =&gt; 21 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;烈雀&quot; , &quot;english&quot; =&gt; &quot;Spearow&quot; , &quot;french&quot; =&gt; &quot;Piafabec&quot; , &quot;japanese&quot; =&gt; &quot;オニスズメ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } } , { 22 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 65 , &quot;HP&quot; =&gt; 65 , &quot;Sp. Attack&quot; =&gt; 61 , &quot;Sp. Defense&quot; =&gt; 61 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 22 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大嘴雀&quot; , &quot;english&quot; =&gt; &quot;Fearow&quot; , &quot;french&quot; =&gt; &quot;Rapasdepic&quot; , &quot;japanese&quot; =&gt; &quot;オニドリル&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } } , { 23 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 44 , &quot;HP&quot; =&gt; 35 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 54 , &quot;Speed&quot; =&gt; 55 } , &quot;id&quot; =&gt; 23 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;阿柏蛇&quot; , &quot;english&quot; =&gt; &quot;Ekans&quot; , &quot;french&quot; =&gt; &quot;Abo&quot; , &quot;japanese&quot; =&gt; &quot;アーボ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } } , { 24 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 95 , &quot;Defense&quot; =&gt; 69 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 79 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 24 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;阿柏怪&quot; , &quot;english&quot; =&gt; &quot;Arbok&quot; , &quot;french&quot; =&gt; &quot;Arbok&quot; , &quot;japanese&quot; =&gt; &quot;アーボック&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } } , { 25 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 55 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 35 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 90 } , &quot;id&quot; =&gt; 25 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮卡丘&quot; , &quot;english&quot; =&gt; &quot;Pikachu&quot; , &quot;french&quot; =&gt; &quot;Pikachu&quot; , &quot;japanese&quot; =&gt; &quot;ピカチュウ&quot; } , &quot;type&quot; =&gt; [ &quot;Electric&quot; ] } } , { 26 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 90 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 110 } , &quot;id&quot; =&gt; 26 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;雷丘&quot; , &quot;english&quot; =&gt; &quot;Raichu&quot; , &quot;french&quot; =&gt; &quot;Raichu&quot; , &quot;japanese&quot; =&gt; &quot;ライチュウ&quot; } , &quot;type&quot; =&gt; [ &quot;Electric&quot; ] } } , { 27 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 75 , &quot;Defense&quot; =&gt; 85 , &quot;HP&quot; =&gt; 50 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 30 , &quot;Speed&quot; =&gt; 40 } , &quot;id&quot; =&gt; 27 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;穿山鼠&quot; , &quot;english&quot; =&gt; &quot;Sandshrew&quot; , &quot;french&quot; =&gt; &quot;Sabelette&quot; , &quot;japanese&quot; =&gt; &quot;サンド&quot; } , &quot;type&quot; =&gt; [ &quot;Ground&quot; ] } } , { 28 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 100 , &quot;Defense&quot; =&gt; 110 , &quot;HP&quot; =&gt; 75 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 28 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;穿山王&quot; , &quot;english&quot; =&gt; &quot;Sandslash&quot; , &quot;french&quot; =&gt; &quot;Sablaireau&quot; , &quot;japanese&quot; =&gt; &quot;サンドパン&quot; } , &quot;type&quot; =&gt; [ &quot;Ground&quot; ] } } , { 29 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 47 , &quot;Defense&quot; =&gt; 52 , &quot;HP&quot; =&gt; 55 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 41 } , &quot;id&quot; =&gt; 29 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多兰&quot; , &quot;english&quot; =&gt; &quot;Nidoran♀&quot; , &quot;french&quot; =&gt; &quot;Nidoran♀&quot; , &quot;japanese&quot; =&gt; &quot;ニドラン♀&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } } , { 30 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 62 , &quot;Defense&quot; =&gt; 67 , &quot;HP&quot; =&gt; 70 , &quot;Sp. Attack&quot; =&gt; 55 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 56 } , &quot;id&quot; =&gt; 30 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多娜&quot; , &quot;english&quot; =&gt; &quot;Nidorina&quot; , &quot;french&quot; =&gt; &quot;Nidorina&quot; , &quot;japanese&quot; =&gt; &quot;ニドリーナ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } } , { 31 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 92 , &quot;Defense&quot; =&gt; 87 , &quot;HP&quot; =&gt; 90 , &quot;Sp. Attack&quot; =&gt; 75 , &quot;Sp. Defense&quot; =&gt; 85 , &quot;Speed&quot; =&gt; 76 } , &quot;id&quot; =&gt; 31 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多后&quot; , &quot;english&quot; =&gt; &quot;Nidoqueen&quot; , &quot;french&quot; =&gt; &quot;Nidoqueen&quot; , &quot;japanese&quot; =&gt; &quot;ニドクイン&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Ground&quot; ] } } , { 32 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 57 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 46 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 50 } , &quot;id&quot; =&gt; 32 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多朗&quot; , &quot;english&quot; =&gt; &quot;Nidoran♂&quot; , &quot;french&quot; =&gt; &quot;Nidoran♂&quot; , &quot;japanese&quot; =&gt; &quot;ニドラン♂&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } } , { 33 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 72 , &quot;Defense&quot; =&gt; 57 , &quot;HP&quot; =&gt; 61 , &quot;Sp. Attack&quot; =&gt; 55 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 33 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多力诺&quot; , &quot;english&quot; =&gt; &quot;Nidorino&quot; , &quot;french&quot; =&gt; &quot;Nidorino&quot; , &quot;japanese&quot; =&gt; &quot;ニドリーノ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } } , { 34 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 102 , &quot;Defense&quot; =&gt; 77 , &quot;HP&quot; =&gt; 81 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 75 , &quot;Speed&quot; =&gt; 85 } , &quot;id&quot; =&gt; 34 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多王&quot; , &quot;english&quot; =&gt; &quot;Nidoking&quot; , &quot;french&quot; =&gt; &quot;Nidoking&quot; , &quot;japanese&quot; =&gt; &quot;ニドキング&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Ground&quot; ] } } , { 35 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 48 , &quot;HP&quot; =&gt; 70 , &quot;Sp. Attack&quot; =&gt; 60 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 35 } , &quot;id&quot; =&gt; 35 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮皮&quot; , &quot;english&quot; =&gt; &quot;Clefairy&quot; , &quot;french&quot; =&gt; &quot;Mélofée&quot; , &quot;japanese&quot; =&gt; &quot;ピッピ&quot; } , &quot;type&quot; =&gt; [ &quot;Fairy&quot; ] } } , { 36 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , &quot;Defense&quot; =&gt; 73 , &quot;HP&quot; =&gt; 95 , &quot;Sp. Attack&quot; =&gt; 95 , &quot;Sp. Defense&quot; =&gt; 90 , &quot;Speed&quot; =&gt; 60 } , &quot;id&quot; =&gt; 36 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮可西&quot; , &quot;english&quot; =&gt; &quot;Clefable&quot; , &quot;french&quot; =&gt; &quot;Mélodelfe&quot; , &quot;japanese&quot; =&gt; &quot;ピクシー&quot; } , &quot;type&quot; =&gt; [ &quot;Fairy&quot; ] } } , { 37 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 41 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 38 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 37 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;六尾&quot; , &quot;english&quot; =&gt; &quot;Vulpix&quot; , &quot;french&quot; =&gt; &quot;Goupix&quot; , &quot;japanese&quot; =&gt; &quot;ロコン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } } , { 38 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 76 , &quot;Defense&quot; =&gt; 75 , &quot;HP&quot; =&gt; 73 , &quot;Sp. Attack&quot; =&gt; 81 , &quot;Sp. Defense&quot; =&gt; 100 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 38 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;九尾&quot; , &quot;english&quot; =&gt; &quot;Ninetales&quot; , &quot;french&quot; =&gt; &quot;Feunard&quot; , &quot;japanese&quot; =&gt; &quot;キュウコン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } } , { 39 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 20 , &quot;HP&quot; =&gt; 115 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 20 } , &quot;id&quot; =&gt; 39 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;胖丁&quot; , &quot;english&quot; =&gt; &quot;Jigglypuff&quot; , &quot;french&quot; =&gt; &quot;Rondoudou&quot; , &quot;japanese&quot; =&gt; &quot;プリン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Fairy&quot; ] } } , { 40 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , &quot;Defense&quot; =&gt; 45 , &quot;HP&quot; =&gt; 140 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 40 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;胖可丁&quot; , &quot;english&quot; =&gt; &quot;Wigglytuff&quot; , &quot;french&quot; =&gt; &quot;Grodoudou&quot; , &quot;japanese&quot; =&gt; &quot;プクリン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Fairy&quot; ] } } , { 41 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 30 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 55 } , &quot;id&quot; =&gt; 41 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;超音蝠&quot; , &quot;english&quot; =&gt; &quot;Zubat&quot; , &quot;french&quot; =&gt; &quot;Nosferapti&quot; , &quot;japanese&quot; =&gt; &quot;ズバット&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Flying&quot; ] } } , { 42 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 70 , &quot;HP&quot; =&gt; 75 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 75 , ... } , &quot;id&quot; =&gt; 42 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大嘴蝠&quot; , &quot;english&quot; =&gt; &quot;Golbat&quot; , &quot;french&quot; =&gt; &quot;Nosferalto&quot; , ... } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Flying&quot; ] } } , { 43 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 50 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 75 , ... } , &quot;id&quot; =&gt; 43 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;走路草&quot; , &quot;english&quot; =&gt; &quot;Oddish&quot; , ... } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , ... ] } } , { 44 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 65 , &quot;Defense&quot; =&gt; 70 , &quot;HP&quot; =&gt; 60 , ... } , &quot;id&quot; =&gt; 44 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;臭臭花&quot; , ... } , &quot;type&quot; =&gt; [ ... ] } } , { 45 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 85 , ... } , &quot;id&quot; =&gt; 45 , &quot;name&quot; =&gt; %{ ... } , ... } } , { 46 , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , ... } , &quot;id&quot; =&gt; 46 , ... } } , { 47 , %{ &quot;base&quot; =&gt; %{ ... } , ... } } , { 48 , %{ ... } } , { 49 , ... } , { ... } , ... ] Now we're ready to insert these records into an :ets table we'll call the Pokedex (a Pokédex is the in-game term for a registry of known Pokémon). We'll make a new table of type :set , to indicate that each object is unique by key: pokedex = :ets . new ( Pokedex , [ :set ] ) :ets . insert ( pokedex , pokedex_objects ) IO . puts ( &quot;Loaded \#{ :ets . info ( pokedex , :size ) } pokémon into the `Pokedex` table.&quot; ) Loaded 809 pokémon into the ` Pokedex ` table . :ok","ref":"guide-tables.html#seeding-some-data"},{"type":"extras","title":"...for ETS/DETS/Mnesia - Mapping And Filtering Without Matcha","doc":"Let's start off using the basic :ets APIs to retrieve data from our table. The :ets.lookup/2 function lets us get our objects by their key — the first element of our tuples. For example, we can check out what the 500th Pokémon is: :ets . lookup ( pokedex , 500 ) |&gt; List . first ( ) |&gt; elem ( 1 ) |&gt; get_in ( [ &quot;name&quot; , &quot;english&quot; ] ) &quot;Emboar&quot; The 500th Pokémon appears to be some creature called Emboar. This scares and confuses me, as I am an elder millenial who never moved on from the 1st generation of 151 Pokémon, and refuse to believe anything has changed from my childhood. How might we go about getting only Pokémon from the first generation, to soothe my anxiety? While we can use simple :ets APIs to get specific objects by key, there isn't an easy way to ask it to give us all objects with a key less than 152... Filtering We could, of course, load all of our table's data into memory, and filter out later generation pokemon there: first_gen_pokemon = :ets . tab2list ( pokedex ) |&gt; Enum . filter ( fn { id , _pokemon } -&gt; id &gt;= 1 and id &lt;= 151 end ) |&gt; Enum . sort_by ( &amp; elem ( &amp;1 , 0 ) ) |&gt; Enum . map ( &amp; elem ( &amp;1 , 1 ) ) [ %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 49 , &quot;Defense&quot; =&gt; 49 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 1 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙种子&quot; , &quot;english&quot; =&gt; &quot;Bulbasaur&quot; , &quot;french&quot; =&gt; &quot;Bulbizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギダネ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 62 , &quot;Defense&quot; =&gt; 63 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 80 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 60 } , &quot;id&quot; =&gt; 2 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙草&quot; , &quot;english&quot; =&gt; &quot;Ivysaur&quot; , &quot;french&quot; =&gt; &quot;Herbizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギソウ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 82 , &quot;Defense&quot; =&gt; 83 , &quot;HP&quot; =&gt; 80 , &quot;Sp. Attack&quot; =&gt; 100 , &quot;Sp. Defense&quot; =&gt; 100 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 3 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;妙蛙花&quot; , &quot;english&quot; =&gt; &quot;Venusaur&quot; , &quot;french&quot; =&gt; &quot;Florizarre&quot; , &quot;japanese&quot; =&gt; &quot;フシギバナ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 52 , &quot;Defense&quot; =&gt; 43 , &quot;HP&quot; =&gt; 39 , &quot;Sp. Attack&quot; =&gt; 60 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 4 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;小火龙&quot; , &quot;english&quot; =&gt; &quot;Charmander&quot; , &quot;french&quot; =&gt; &quot;Salamèche&quot; , &quot;japanese&quot; =&gt; &quot;ヒトカゲ&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 64 , &quot;Defense&quot; =&gt; 58 , &quot;HP&quot; =&gt; 58 , &quot;Sp. Attack&quot; =&gt; 80 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 5 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;火恐龙&quot; , &quot;english&quot; =&gt; &quot;Charmeleon&quot; , &quot;french&quot; =&gt; &quot;Reptincel&quot; , &quot;japanese&quot; =&gt; &quot;リザード&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 84 , &quot;Defense&quot; =&gt; 78 , &quot;HP&quot; =&gt; 78 , &quot;Sp. Attack&quot; =&gt; 109 , &quot;Sp. Defense&quot; =&gt; 85 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 6 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;喷火龙&quot; , &quot;english&quot; =&gt; &quot;Charizard&quot; , &quot;french&quot; =&gt; &quot;Dracaufeu&quot; , &quot;japanese&quot; =&gt; &quot;リザードン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 48 , &quot;Defense&quot; =&gt; 65 , &quot;HP&quot; =&gt; 44 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 64 , &quot;Speed&quot; =&gt; 43 } , &quot;id&quot; =&gt; 7 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;杰尼龟&quot; , &quot;english&quot; =&gt; &quot;Squirtle&quot; , &quot;french&quot; =&gt; &quot;Carapuce&quot; , &quot;japanese&quot; =&gt; &quot;ゼニガメ&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 63 , &quot;Defense&quot; =&gt; 80 , &quot;HP&quot; =&gt; 59 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 58 } , &quot;id&quot; =&gt; 8 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;卡咪龟&quot; , &quot;english&quot; =&gt; &quot;Wartortle&quot; , &quot;french&quot; =&gt; &quot;Carabaffe&quot; , &quot;japanese&quot; =&gt; &quot;カメール&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 83 , &quot;Defense&quot; =&gt; 100 , &quot;HP&quot; =&gt; 79 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 105 , &quot;Speed&quot; =&gt; 78 } , &quot;id&quot; =&gt; 9 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;水箭龟&quot; , &quot;english&quot; =&gt; &quot;Blastoise&quot; , &quot;french&quot; =&gt; &quot;Tortank&quot; , &quot;japanese&quot; =&gt; &quot;カメックス&quot; } , &quot;type&quot; =&gt; [ &quot;Water&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 30 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 20 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 10 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;绿毛虫&quot; , &quot;english&quot; =&gt; &quot;Caterpie&quot; , &quot;french&quot; =&gt; &quot;Chenipan&quot; , &quot;japanese&quot; =&gt; &quot;キャタピー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 20 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 50 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 30 } , &quot;id&quot; =&gt; 11 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;铁甲蛹&quot; , &quot;english&quot; =&gt; &quot;Metapod&quot; , &quot;french&quot; =&gt; &quot;Chrysacier&quot; , &quot;japanese&quot; =&gt; &quot;トランセル&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 50 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 90 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 70 } , &quot;id&quot; =&gt; 12 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;巴大蝶&quot; , &quot;english&quot; =&gt; &quot;Butterfree&quot; , &quot;french&quot; =&gt; &quot;Papilusion&quot; , &quot;japanese&quot; =&gt; &quot;バタフリー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 35 , &quot;Defense&quot; =&gt; 30 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 20 , &quot;Speed&quot; =&gt; 50 } , &quot;id&quot; =&gt; 13 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;独角虫&quot; , &quot;english&quot; =&gt; &quot;Weedle&quot; , &quot;french&quot; =&gt; &quot;Aspicot&quot; , &quot;japanese&quot; =&gt; &quot;ビードル&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 25 , &quot;Defense&quot; =&gt; 50 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 35 } , &quot;id&quot; =&gt; 14 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;铁壳蛹&quot; , &quot;english&quot; =&gt; &quot;Kakuna&quot; , &quot;french&quot; =&gt; &quot;Coconfort&quot; , &quot;japanese&quot; =&gt; &quot;コクーン&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 65 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 75 } , &quot;id&quot; =&gt; 15 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大针蜂&quot; , &quot;english&quot; =&gt; &quot;Beedrill&quot; , &quot;french&quot; =&gt; &quot;Dardargnan&quot; , &quot;japanese&quot; =&gt; &quot;スピアー&quot; } , &quot;type&quot; =&gt; [ &quot;Bug&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 35 , &quot;Sp. Defense&quot; =&gt; 35 , &quot;Speed&quot; =&gt; 56 } , &quot;id&quot; =&gt; 16 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;波波&quot; , &quot;english&quot; =&gt; &quot;Pidgey&quot; , &quot;french&quot; =&gt; &quot;Roucool&quot; , &quot;japanese&quot; =&gt; &quot;ポッポ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 63 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 71 } , &quot;id&quot; =&gt; 17 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;比比鸟&quot; , &quot;english&quot; =&gt; &quot;Pidgeotto&quot; , &quot;french&quot; =&gt; &quot;Roucoups&quot; , &quot;japanese&quot; =&gt; &quot;ピジョン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 75 , &quot;HP&quot; =&gt; 83 , &quot;Sp. Attack&quot; =&gt; 70 , &quot;Sp. Defense&quot; =&gt; 70 , &quot;Speed&quot; =&gt; 101 } , &quot;id&quot; =&gt; 18 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大比鸟&quot; , &quot;english&quot; =&gt; &quot;Pidgeot&quot; , &quot;french&quot; =&gt; &quot;Roucarnage&quot; , &quot;japanese&quot; =&gt; &quot;ピジョット&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 56 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 30 , &quot;Sp. Attack&quot; =&gt; 25 , &quot;Sp. Defense&quot; =&gt; 35 , &quot;Speed&quot; =&gt; 72 } , &quot;id&quot; =&gt; 19 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;小拉达&quot; , &quot;english&quot; =&gt; &quot;Rattata&quot; , &quot;french&quot; =&gt; &quot;Rattata&quot; , &quot;japanese&quot; =&gt; &quot;コラッタ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 81 , &quot;Defense&quot; =&gt; 60 , &quot;HP&quot; =&gt; 55 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 70 , &quot;Speed&quot; =&gt; 97 } , &quot;id&quot; =&gt; 20 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;拉达&quot; , &quot;english&quot; =&gt; &quot;Raticate&quot; , &quot;french&quot; =&gt; &quot;Rattatac&quot; , &quot;japanese&quot; =&gt; &quot;ラッタ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 30 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 31 , &quot;Sp. Defense&quot; =&gt; 31 , &quot;Speed&quot; =&gt; 70 } , &quot;id&quot; =&gt; 21 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;烈雀&quot; , &quot;english&quot; =&gt; &quot;Spearow&quot; , &quot;french&quot; =&gt; &quot;Piafabec&quot; , &quot;japanese&quot; =&gt; &quot;オニスズメ&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 65 , &quot;HP&quot; =&gt; 65 , &quot;Sp. Attack&quot; =&gt; 61 , &quot;Sp. Defense&quot; =&gt; 61 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 22 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大嘴雀&quot; , &quot;english&quot; =&gt; &quot;Fearow&quot; , &quot;french&quot; =&gt; &quot;Rapasdepic&quot; , &quot;japanese&quot; =&gt; &quot;オニドリル&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 60 , &quot;Defense&quot; =&gt; 44 , &quot;HP&quot; =&gt; 35 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 54 , &quot;Speed&quot; =&gt; 55 } , &quot;id&quot; =&gt; 23 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;阿柏蛇&quot; , &quot;english&quot; =&gt; &quot;Ekans&quot; , &quot;french&quot; =&gt; &quot;Abo&quot; , &quot;japanese&quot; =&gt; &quot;アーボ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 95 , &quot;Defense&quot; =&gt; 69 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 79 , &quot;Speed&quot; =&gt; 80 } , &quot;id&quot; =&gt; 24 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;阿柏怪&quot; , &quot;english&quot; =&gt; &quot;Arbok&quot; , &quot;french&quot; =&gt; &quot;Arbok&quot; , &quot;japanese&quot; =&gt; &quot;アーボック&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 55 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 35 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 90 } , &quot;id&quot; =&gt; 25 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮卡丘&quot; , &quot;english&quot; =&gt; &quot;Pikachu&quot; , &quot;french&quot; =&gt; &quot;Pikachu&quot; , &quot;japanese&quot; =&gt; &quot;ピカチュウ&quot; } , &quot;type&quot; =&gt; [ &quot;Electric&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 90 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 90 , &quot;Sp. Defense&quot; =&gt; 80 , &quot;Speed&quot; =&gt; 110 } , &quot;id&quot; =&gt; 26 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;雷丘&quot; , &quot;english&quot; =&gt; &quot;Raichu&quot; , &quot;french&quot; =&gt; &quot;Raichu&quot; , &quot;japanese&quot; =&gt; &quot;ライチュウ&quot; } , &quot;type&quot; =&gt; [ &quot;Electric&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 75 , &quot;Defense&quot; =&gt; 85 , &quot;HP&quot; =&gt; 50 , &quot;Sp. Attack&quot; =&gt; 20 , &quot;Sp. Defense&quot; =&gt; 30 , &quot;Speed&quot; =&gt; 40 } , &quot;id&quot; =&gt; 27 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;穿山鼠&quot; , &quot;english&quot; =&gt; &quot;Sandshrew&quot; , &quot;french&quot; =&gt; &quot;Sabelette&quot; , &quot;japanese&quot; =&gt; &quot;サンド&quot; } , &quot;type&quot; =&gt; [ &quot;Ground&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 100 , &quot;Defense&quot; =&gt; 110 , &quot;HP&quot; =&gt; 75 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 28 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;穿山王&quot; , &quot;english&quot; =&gt; &quot;Sandslash&quot; , &quot;french&quot; =&gt; &quot;Sablaireau&quot; , &quot;japanese&quot; =&gt; &quot;サンドパン&quot; } , &quot;type&quot; =&gt; [ &quot;Ground&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 47 , &quot;Defense&quot; =&gt; 52 , &quot;HP&quot; =&gt; 55 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 41 } , &quot;id&quot; =&gt; 29 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多兰&quot; , &quot;english&quot; =&gt; &quot;Nidoran♀&quot; , &quot;french&quot; =&gt; &quot;Nidoran♀&quot; , &quot;japanese&quot; =&gt; &quot;ニドラン♀&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 62 , &quot;Defense&quot; =&gt; 67 , &quot;HP&quot; =&gt; 70 , &quot;Sp. Attack&quot; =&gt; 55 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 56 } , &quot;id&quot; =&gt; 30 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多娜&quot; , &quot;english&quot; =&gt; &quot;Nidorina&quot; , &quot;french&quot; =&gt; &quot;Nidorina&quot; , &quot;japanese&quot; =&gt; &quot;ニドリーナ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 92 , &quot;Defense&quot; =&gt; 87 , &quot;HP&quot; =&gt; 90 , &quot;Sp. Attack&quot; =&gt; 75 , &quot;Sp. Defense&quot; =&gt; 85 , &quot;Speed&quot; =&gt; 76 } , &quot;id&quot; =&gt; 31 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多后&quot; , &quot;english&quot; =&gt; &quot;Nidoqueen&quot; , &quot;french&quot; =&gt; &quot;Nidoqueen&quot; , &quot;japanese&quot; =&gt; &quot;ニドクイン&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Ground&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 57 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 46 , &quot;Sp. Attack&quot; =&gt; 40 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 50 } , &quot;id&quot; =&gt; 32 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多朗&quot; , &quot;english&quot; =&gt; &quot;Nidoran♂&quot; , &quot;french&quot; =&gt; &quot;Nidoran♂&quot; , &quot;japanese&quot; =&gt; &quot;ニドラン♂&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 72 , &quot;Defense&quot; =&gt; 57 , &quot;HP&quot; =&gt; 61 , &quot;Sp. Attack&quot; =&gt; 55 , &quot;Sp. Defense&quot; =&gt; 55 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 33 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多力诺&quot; , &quot;english&quot; =&gt; &quot;Nidorino&quot; , &quot;french&quot; =&gt; &quot;Nidorino&quot; , &quot;japanese&quot; =&gt; &quot;ニドリーノ&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 102 , &quot;Defense&quot; =&gt; 77 , &quot;HP&quot; =&gt; 81 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 75 , &quot;Speed&quot; =&gt; 85 } , &quot;id&quot; =&gt; 34 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;尼多王&quot; , &quot;english&quot; =&gt; &quot;Nidoking&quot; , &quot;french&quot; =&gt; &quot;Nidoking&quot; , &quot;japanese&quot; =&gt; &quot;ニドキング&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Ground&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 48 , &quot;HP&quot; =&gt; 70 , &quot;Sp. Attack&quot; =&gt; 60 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 35 } , &quot;id&quot; =&gt; 35 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮皮&quot; , &quot;english&quot; =&gt; &quot;Clefairy&quot; , &quot;french&quot; =&gt; &quot;Mélofée&quot; , &quot;japanese&quot; =&gt; &quot;ピッピ&quot; } , &quot;type&quot; =&gt; [ &quot;Fairy&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , &quot;Defense&quot; =&gt; 73 , &quot;HP&quot; =&gt; 95 , &quot;Sp. Attack&quot; =&gt; 95 , &quot;Sp. Defense&quot; =&gt; 90 , &quot;Speed&quot; =&gt; 60 } , &quot;id&quot; =&gt; 36 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;皮可西&quot; , &quot;english&quot; =&gt; &quot;Clefable&quot; , &quot;french&quot; =&gt; &quot;Mélodelfe&quot; , &quot;japanese&quot; =&gt; &quot;ピクシー&quot; } , &quot;type&quot; =&gt; [ &quot;Fairy&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 41 , &quot;Defense&quot; =&gt; 40 , &quot;HP&quot; =&gt; 38 , &quot;Sp. Attack&quot; =&gt; 50 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 65 } , &quot;id&quot; =&gt; 37 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;六尾&quot; , &quot;english&quot; =&gt; &quot;Vulpix&quot; , &quot;french&quot; =&gt; &quot;Goupix&quot; , &quot;japanese&quot; =&gt; &quot;ロコン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 76 , &quot;Defense&quot; =&gt; 75 , &quot;HP&quot; =&gt; 73 , &quot;Sp. Attack&quot; =&gt; 81 , &quot;Sp. Defense&quot; =&gt; 100 , &quot;Speed&quot; =&gt; 100 } , &quot;id&quot; =&gt; 38 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;九尾&quot; , &quot;english&quot; =&gt; &quot;Ninetales&quot; , &quot;french&quot; =&gt; &quot;Feunard&quot; , &quot;japanese&quot; =&gt; &quot;キュウコン&quot; } , &quot;type&quot; =&gt; [ &quot;Fire&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 20 , &quot;HP&quot; =&gt; 115 , &quot;Sp. Attack&quot; =&gt; 45 , &quot;Sp. Defense&quot; =&gt; 25 , &quot;Speed&quot; =&gt; 20 } , &quot;id&quot; =&gt; 39 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;胖丁&quot; , &quot;english&quot; =&gt; &quot;Jigglypuff&quot; , &quot;french&quot; =&gt; &quot;Rondoudou&quot; , &quot;japanese&quot; =&gt; &quot;プリン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Fairy&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , &quot;Defense&quot; =&gt; 45 , &quot;HP&quot; =&gt; 140 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 50 , &quot;Speed&quot; =&gt; 45 } , &quot;id&quot; =&gt; 40 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;胖可丁&quot; , &quot;english&quot; =&gt; &quot;Wigglytuff&quot; , &quot;french&quot; =&gt; &quot;Grodoudou&quot; , &quot;japanese&quot; =&gt; &quot;プクリン&quot; } , &quot;type&quot; =&gt; [ &quot;Normal&quot; , &quot;Fairy&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 45 , &quot;Defense&quot; =&gt; 35 , &quot;HP&quot; =&gt; 40 , &quot;Sp. Attack&quot; =&gt; 30 , &quot;Sp. Defense&quot; =&gt; 40 , &quot;Speed&quot; =&gt; 55 } , &quot;id&quot; =&gt; 41 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;超音蝠&quot; , &quot;english&quot; =&gt; &quot;Zubat&quot; , &quot;french&quot; =&gt; &quot;Nosferapti&quot; , &quot;japanese&quot; =&gt; &quot;ズバット&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 70 , &quot;HP&quot; =&gt; 75 , &quot;Sp. Attack&quot; =&gt; 65 , &quot;Sp. Defense&quot; =&gt; 75 , &quot;Speed&quot; =&gt; 90 } , &quot;id&quot; =&gt; 42 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;大嘴蝠&quot; , &quot;english&quot; =&gt; &quot;Golbat&quot; , &quot;french&quot; =&gt; &quot;Nosferalto&quot; , &quot;japanese&quot; =&gt; &quot;ゴルバット&quot; } , &quot;type&quot; =&gt; [ &quot;Poison&quot; , &quot;Flying&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 50 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 45 , &quot;Sp. Attack&quot; =&gt; 75 , &quot;Sp. Defense&quot; =&gt; 65 , &quot;Speed&quot; =&gt; 30 } , &quot;id&quot; =&gt; 43 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;走路草&quot; , &quot;english&quot; =&gt; &quot;Oddish&quot; , &quot;french&quot; =&gt; &quot;Mystherbe&quot; , &quot;japanese&quot; =&gt; &quot;ナゾノクサ&quot; } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 65 , &quot;Defense&quot; =&gt; 70 , &quot;HP&quot; =&gt; 60 , &quot;Sp. Attack&quot; =&gt; 85 , &quot;Sp. Defense&quot; =&gt; 75 , ... } , &quot;id&quot; =&gt; 44 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;臭臭花&quot; , &quot;english&quot; =&gt; &quot;Gloom&quot; , &quot;french&quot; =&gt; &quot;Ortide&quot; , ... } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , &quot;Poison&quot; ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 80 , &quot;Defense&quot; =&gt; 85 , &quot;HP&quot; =&gt; 75 , &quot;Sp. Attack&quot; =&gt; 110 , ... } , &quot;id&quot; =&gt; 45 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;霸王花&quot; , &quot;english&quot; =&gt; &quot;Vileplume&quot; , ... } , &quot;type&quot; =&gt; [ &quot;Grass&quot; , ... ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 70 , &quot;Defense&quot; =&gt; 55 , &quot;HP&quot; =&gt; 35 , ... } , &quot;id&quot; =&gt; 46 , &quot;name&quot; =&gt; %{ &quot;chinese&quot; =&gt; &quot;派拉斯&quot; , ... } , &quot;type&quot; =&gt; [ ... ] } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 95 , &quot;Defense&quot; =&gt; 80 , ... } , &quot;id&quot; =&gt; 47 , &quot;name&quot; =&gt; %{ ... } , ... } , %{ &quot;base&quot; =&gt; %{ &quot;Attack&quot; =&gt; 55 , ... } , &quot;id&quot; =&gt; 48 , ... } , %{ &quot;base&quot; =&gt; %{ ... } , ... } , %{ ... } , ... ] Here, we're using :ets.tab2list/1 to load the entire :ets dataset into our process's memory. This filtering succeeds in giving us only the 151 Pokémon I am personally comfortable admitting exist: first_gen_pokemon |&gt; length 151 Mapping Let's also try extracting just the names of the Pokémon we know about: pokemon_names = :ets . tab2list ( pokedex ) |&gt; Enum . map ( fn { _id , %{ &quot;name&quot; =&gt; %{ &quot;english&quot; =&gt; name } } } -&gt; name end ) [ &quot;Melmetal&quot; , &quot;Lugia&quot; , &quot;Misdreavus&quot; , &quot;Clefairy&quot; , &quot;Swirlix&quot; , &quot;Absol&quot; , &quot;Tapu Bulu&quot; , &quot;Nidoqueen&quot; , &quot;Venusaur&quot; , &quot;Meowth&quot; , &quot;Camerupt&quot; , &quot;Wobbuffet&quot; , &quot;Tapu Lele&quot; , &quot;Tynamo&quot; , &quot;Decidueye&quot; , &quot;Wailmer&quot; , &quot;Bibarel&quot; , &quot;Solrock&quot; , &quot;Meganium&quot; , &quot;Dusclops&quot; , &quot;Mareep&quot; , &quot;Manaphy&quot; , &quot;Chinchou&quot; , &quot;Breloom&quot; , &quot;Smoochum&quot; , &quot;Corphish&quot; , &quot;Dragonair&quot; , &quot;Feraligatr&quot; , &quot;Zebstrika&quot; , &quot;Staravia&quot; , &quot;Snubbull&quot; , &quot;Espurr&quot; , &quot;Krabby&quot; , &quot;Lucario&quot; , &quot;Cloyster&quot; , &quot;Igglybuff&quot; , &quot;Xatu&quot; , &quot;Exeggcute&quot; , &quot;Tangela&quot; , &quot;Marshadow&quot; , &quot;Sharpedo&quot; , &quot;Floatzel&quot; , &quot;Infernape&quot; , &quot;Heatran&quot; , &quot;Tentacool&quot; , &quot;Elgyem&quot; , &quot;Haxorus&quot; , &quot;Magneton&quot; , &quot;Vanillite&quot; , &quot;Nosepass&quot; , ... ] The Problem Of course, this is all terribly inefficient. :ets has to load all of its data into our process's memory, where we only want a known handful of it. For large tables, this would greatly harm the performance of our filtering, and even risk crashing the process. Mapping is even more problematic: if we don't do a filter beforehand, we have to map over every object, even the ones we are not interested in. This is generally how :ets is designed to be used, though: storing data we want globally accessible, and looking up known, specific objects by index where and when we need them. :ets in its normal operation can be thought of as a global key/value store for arbitrary terms... That is, until you start using match specs to query them!","ref":"guide-tables.html#mapping-and-filtering-without-matcha"},{"type":"extras","title":"...for ETS/DETS/Mnesia - Filtering and Mapping With Matcha","doc":"Filtering and mapping :ets data efficiently is exactly what match specs were invented to accomplish. We can trivially reproduce the in-memory filtering of our 1st generation of Pokémon with Matcha , using match specs to push the actual work of filtering into :ets itself, with a much more efficient querying mechanism, and copying just the data we want into our process. Filtering A Matcha.Spec used in :ets filtering looks like a case statement, where any object that does not match our patterns is never returned from the table: require Matcha.Table.ETS Matcha.Table.ETS . select pokedex do { id , _pokemon } = object when id in 1 .. 151 -&gt; object end |&gt; length ( ) 151 Mapping Of course, we don't have to just return the full :ets object; we can also destructure matched objects, and extract just the data from them we are interested in: alias Matcha.Table.ETS require ETS ETS . select pokedex do { id , %{ &quot;name&quot; =&gt; %{ &quot;english&quot; =&gt; name } } } when id in 1 .. 151 -&gt; name end [ &quot;Clefairy&quot; , &quot;Nidoqueen&quot; , &quot;Venusaur&quot; , &quot;Meowth&quot; , &quot;Dragonair&quot; , &quot;Krabby&quot; , &quot;Cloyster&quot; , &quot;Exeggcute&quot; , &quot;Tangela&quot; , &quot;Tentacool&quot; , &quot;Magneton&quot; , &quot;Butterfree&quot; , &quot;Omastar&quot; , &quot;Rhydon&quot; , &quot;Persian&quot; , &quot;Aerodactyl&quot; , &quot;Psyduck&quot; , &quot;Ditto&quot; , &quot;Porygon&quot; , &quot;Arbok&quot; , &quot;Flareon&quot; , &quot;Victreebel&quot; , &quot;Fearow&quot; , &quot;Alakazam&quot; , &quot;Raichu&quot; , &quot;Eevee&quot; , &quot;Ivysaur&quot; , &quot;Cubone&quot; , &quot;Hypno&quot; , &quot;Spearow&quot; , &quot;Pinsir&quot; , &quot;Bellsprout&quot; , &quot;Drowzee&quot; , &quot;Jynx&quot; , &quot;Charmander&quot; , &quot;Squirtle&quot; , &quot;Mankey&quot; , &quot;Electabuzz&quot; , &quot;Rhyhorn&quot; , &quot;Pidgey&quot; , &quot;Nidorino&quot; , &quot;Nidoking&quot; , &quot;Raticate&quot; , &quot;Jolteon&quot; , &quot;Growlithe&quot; , &quot;Seaking&quot; , &quot;Magikarp&quot; , &quot;Rapidash&quot; , &quot;Sandshrew&quot; , &quot;Muk&quot; , ... ] This is what the &quot;mapping&quot; aspect of match specs refers to: that we can not only filter out objects that do not match our pattern, but transform the data we return from the match into what we are interested in, as if passing it through an Enum.map/2 , except at a much lower level, greatly improving the efficiency of our querying. Multiple Clauses Just as data in a list in Elixir need not be homogenous, but can contain any term; objects in the same :ets table can be any term — just so long as they are all tuples, and all have a key entry at the same position in the tuples: hetrogenous_table = :ets . new ( HetrogenousTable , [ :set ] ) :ets . insert ( hetrogenous_table , [ { 3 , &quot;three&quot; , &quot;tuple&quot; } , { 4 , &quot;four&quot; , &quot;tuple&quot; , &quot;object&quot; } ] ) true Our match specs can use multiple clauses to match on these different object shapes: alias Matcha.Table.ETS require ETS ETS . select hetrogenous_table do { _key , length , _ } -&gt; length { _key , length , _ , _ } -&gt; length end [ &quot;four&quot; , &quot;three&quot; ] We can also return hetrogenous shapes from :ets , our mapping operation need not always shape query results the same: alias Matcha.Table.ETS require ETS ETS . select hetrogenous_table do { _key , _ , _ } -&gt; [ { :three , :object , :shape } ] { _key , _ , _ , _ } -&gt; [ { :four , :object , :shape , :tuple } ] end [ [ { :four , :object , :shape , :tuple } ] , [ { :three , :object , :shape } ] ] Combining this with the specificity and expressivity of pattern matching, we can do some very powerful filter/mapping in a very efficient way: alias Matcha.Table.ETS require ETS ETS . select pokedex do { id , %{ &quot;type&quot; =&gt; types , &quot;name&quot; =&gt; %{ &quot;english&quot; =&gt; name } } } when id in 1 .. 151 -&gt; { name , generation : 1 , types : types } { id , %{ &quot;type&quot; =&gt; types , &quot;name&quot; =&gt; %{ &quot;english&quot; =&gt; name } } } when id in 152 .. 251 -&gt; { name , generation : 2 , types : types } # Other generations were a mistake end [ { &quot;Lugia&quot; , [ generation : 2 , types : [ &quot;Psychic&quot; , &quot;Flying&quot; ] ] } , { &quot;Misdreavus&quot; , [ generation : 2 , types : [ &quot;Ghost&quot; ] ] } , { &quot;Clefairy&quot; , [ generation : 1 , types : [ &quot;Fairy&quot; ] ] } , { &quot;Nidoqueen&quot; , [ generation : 1 , types : [ &quot;Poison&quot; , &quot;Ground&quot; ] ] } , { &quot;Venusaur&quot; , [ generation : 1 , types : [ &quot;Grass&quot; , &quot;Poison&quot; ] ] } , { &quot;Meowth&quot; , [ generation : 1 , types : [ &quot;Normal&quot; ] ] } , { &quot;Wobbuffet&quot; , [ generation : 2 , types : [ &quot;Psychic&quot; ] ] } , { &quot;Meganium&quot; , [ generation : 2 , types : [ &quot;Grass&quot; ] ] } , { &quot;Mareep&quot; , [ generation : 2 , types : [ &quot;Electric&quot; ] ] } , { &quot;Chinchou&quot; , [ generation : 2 , types : [ &quot;Water&quot; , &quot;Electric&quot; ] ] } , { &quot;Smoochum&quot; , [ generation : 2 , types : [ &quot;Ice&quot; , &quot;Psychic&quot; ] ] } , { &quot;Dragonair&quot; , [ generation : 1 , types : [ &quot;Dragon&quot; ] ] } , { &quot;Feraligatr&quot; , [ generation : 2 , types : [ &quot;Water&quot; ] ] } , { &quot;Snubbull&quot; , [ generation : 2 , types : [ &quot;Fairy&quot; ] ] } , { &quot;Krabby&quot; , [ generation : 1 , types : [ &quot;Water&quot; ] ] } , { &quot;Cloyster&quot; , [ generation : 1 , types : [ &quot;Water&quot; , &quot;Ice&quot; ] ] } , { &quot;Igglybuff&quot; , [ generation : 2 , types : [ &quot;Normal&quot; , &quot;Fairy&quot; ] ] } , { &quot;Xatu&quot; , [ generation : 2 , types : [ &quot;Psychic&quot; , &quot;Flying&quot; ] ] } , { &quot;Exeggcute&quot; , [ generation : 1 , types : [ &quot;Grass&quot; , &quot;Psychic&quot; ] ] } , { &quot;Tangela&quot; , [ generation : 1 , types : [ &quot;Grass&quot; ] ] } , { &quot;Tentacool&quot; , [ generation : 1 , types : [ &quot;Water&quot; , &quot;Poison&quot; ] ] } , { &quot;Magneton&quot; , [ generation : 1 , types : [ &quot;Electric&quot; , &quot;Steel&quot; ] ] } , { &quot;Togetic&quot; , [ generation : 2 , types : [ &quot;Fairy&quot; , &quot;Flying&quot; ] ] } , { &quot;Scizor&quot; , [ generation : 2 , types : [ &quot;Bug&quot; , &quot;Steel&quot; ] ] } , { &quot;Cyndaquil&quot; , [ generation : 2 , types : [ &quot;Fire&quot; ] ] } , { &quot;Steelix&quot; , [ generation : 2 , types : [ &quot;Steel&quot; , &quot;Ground&quot; ] ] } , { &quot;Unown&quot; , [ generation : 2 , types : [ &quot;Psychic&quot; ] ] } , { &quot;Magcargo&quot; , [ generation : 2 , types : [ &quot;Fire&quot; , &quot;Rock&quot; ] ] } , { &quot;Butterfree&quot; , [ generation : 1 , types : [ &quot;Bug&quot; , &quot;Flying&quot; ] ] } , { &quot;Entei&quot; , [ generation : 2 , types : [ &quot;Fire&quot; ] ] } , { &quot;Omastar&quot; , [ generation : 1 , types : [ &quot;Rock&quot; , &quot;Water&quot; ] ] } , { &quot;Rhydon&quot; , [ generation : 1 , types : [ &quot;Ground&quot; , &quot;Rock&quot; ] ] } , { &quot;Persian&quot; , [ generation : 1 , types : [ &quot;Normal&quot; ] ] } , { &quot;Aerodactyl&quot; , [ generation : 1 , types : [ &quot;Rock&quot; , &quot;Flying&quot; ] ] } , { &quot;Psyduck&quot; , [ generation : 1 , types : [ &quot;Water&quot; ] ] } , { &quot;Corsola&quot; , [ generation : 2 , types : [ &quot;Water&quot; , &quot;Rock&quot; ] ] } , { &quot;Pineco&quot; , [ generation : 2 , types : [ &quot;Bug&quot; ] ] } , { &quot;Granbull&quot; , [ generation : 2 , types : [ &quot;Fairy&quot; ] ] } , { &quot;Ditto&quot; , [ generation : 1 , types : [ &quot;Normal&quot; ] ] } , { &quot;Bayleef&quot; , [ generation : 2 , types : [ &quot;Grass&quot; ] ] } , { &quot;Porygon&quot; , [ generation : 1 , types : [ &quot;Normal&quot; ] ] } , { &quot;Arbok&quot; , [ generation : 1 , types : [ &quot;Poison&quot; ] ] } , { &quot;Pichu&quot; , [ generation : 2 , types : [ &quot;Electric&quot; ] ] } , { &quot;Flareon&quot; , [ generation : 1 , types : [ &quot;Fire&quot; ] ] } , { &quot;Hoothoot&quot; , [ generation : 2 , types : [ &quot;Normal&quot; , ... ] ] } , { &quot;Victreebel&quot; , [ generation : 1 , types : [ ... ] ] } , { &quot;Remoraid&quot; , [ generation : 2 , ... ] } , { &quot;Tyranitar&quot; , [ ... ] } , { &quot;Fearow&quot; , ... } , { ... } , ... ]","ref":"guide-tables.html#filtering-and-mapping-with-matcha"},{"type":"extras","title":"...for Tracing","doc":"Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } ] , force : true ) IO . puts ( &quot;Installed matcha version: \#{ Application . spec ( :matcha , :vsn ) } &quot; ) Installed matcha version : 0.1 . 4 :ok This is a gentle introduction to tracing in the BEAM VM. For a quick reference, see the tracing cheatsheet .","ref":"guide-tracing.html"},{"type":"extras","title":"...for Tracing - What's Tracing?","doc":"The BEAM VM has the powerful ability to report when (almost) any function in any module is called, from any process in a distributed system, to your current process. This technique is generally referred to as tracing . This is an incredibly useful first-class debugging tool to understand the behaviour of your program at runtime! However, it can be a little difficult to use the lower-level Erlang calls to set this up. For this reason, Matcha offers a higher-level API to the VM's tracing engine. It also makes using matchspecs when tracing trivial, to trace even more precisely. This guide demonstrates various ways to trace calls in our system, how to interpret the results, and how to customize the behaviour of the tracing engine even further.","ref":"guide-tracing.html#what-s-tracing"},{"type":"extras","title":"...for Tracing - Trace Messages","doc":"If we want to know when any function in the Integer module is called, we can trace it: Matcha.Trace . module ( Integer ) For example, evaluating this code will send us a message the next time any Integer function is called: Matcha.Trace . module ( Integer ) Integer . parse ( &quot;1&quot; )","ref":"guide-tracing.html#trace-messages"},{"type":"extras","title":"...for Tracing - Limiting Trace Messages","doc":"As with all powerful things, this must be used responsibly—tracing too much in a busy system could easily send way too many messages to our current process and crash it. Best case scenario, this would crash a process we'd connected to a running system to inspect it. Worst case, the process receiving trace messages is a load-bearing part of our system, and could cause cascading failures! This is why Matcha wraps lower-level tracing calls (via :recon ) with a higher-level API that limits how many messages we may receive from the tracing engine. You'll notice after we traced one Integer call above, we were informed that our default rate limter halted tracing automatically for us. This ensures that we cannot shoot ourselves in the foot; for example the second call here does not get traced: Matcha.Trace . module ( Integer ) Integer . parse ( &quot;1&quot; ) # This second call will not be traced! Integer . parse ( &quot;2&quot; ) If we want to trace more than just one call, we can specify a :limit to our messages between 0 and 255 : Matcha.Trace . module ( Integer , limit : 2 ) Integer . parse ( &quot;1&quot; ) Integer . parse ( &quot;2&quot; ) # This third call will not be traced! Integer . parse ( &quot;3&quot; ) Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Recon tracer rate limit tripped . { 3 , &quot;&quot; } Note that these limits operate on the number of messages received, not the number of calls traced. You may configure a trace to emit more than one message; each one will count against the limit. Another way to trace safer, or just to be more specific about what you're interested in getting messages for, is to be more specific about what you want to be notified about. Let's explore how we can refine our traces!","ref":"guide-tracing.html#limiting-trace-messages"},{"type":"extras","title":"...for Tracing - Tracing Entire Modules","doc":"As we've already demonstrated, you can trace entire modules, getting notified when any public function within is called. Matcha.Trace . module ( Integer , limit : 2 ) Integer . digits ( 123 ) Integer . parse ( &quot;1&quot; ) Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . parse ( &quot;2&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Recon tracer rate limit tripped . { 1 , &quot;&quot; } Odds are, the functions we are interested in tracing are probably some of our core functions. However, trying to trace these very frequently invoked functions at the entire module level is going to produce a lot of messages. Either we are going to trip our rate limiter before we see the functions we are interested in getting traced, or we are going to have to up our message limit substantially and dig through a whole lot to see what we're interested in. Fortunately, we can be increasingly specific about what we're interested in tracing!","ref":"guide-tracing.html#tracing-entire-modules"},{"type":"extras","title":"...for Tracing - Tracing Functions","doc":"The next level of specificity tracing supports is tracing a specific function. To continue our example, maybe we are specifically interested in calls to Integer.parse alone. We can just trace that function: Matcha.Trace . function ( Integer , :parse ) Integer . parse ( &quot;1&quot; ) Matcha.Trace : ` Elixir.Integer . digits ( 123 ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Recon tracer rate limit tripped . { 1 , &quot;&quot; } This lets us ignore other functions called to the Integer module, so other code that executes doesn't accidentally trip our rate limiter before getting to the code we are interested in: Matcha.Trace . function ( Integer , :parse ) # Will no longer be traced Integer . digits ( 123 ) Integer . parse ( &quot;1&quot; ) Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Recon tracer rate limit tripped . { 1 , &quot;&quot; } In these simple scripted examples, this may feel like overkill—but when connected to a busy running system, it's quite likely that functions we're not interested from the module we are interested in will get called. Specifying exactly which functions interest us prevents that. However, we can get even more specific.","ref":"guide-tracing.html#tracing-functions"},{"type":"extras","title":"...for Tracing - Tracing Specific Arities","doc":"A function's arity describes the number of arguments it takes. Since Elixir supports varadic functions, functions with different implementations based on the number of arguments given, filtering trace messages by arity gives us a way of being even more specific about what we're interested in. For example, most Integer functions take an optional base argument, defaulting to 10 since decimal is most commonly the base we intend integers to represent. If we are only interested in cases where we are parsing a string into an integer with the non-default decimal base, we can trace calls to Integer.parse with an arity of 2 —that is, functions were we are specifying an extra base argument: # Important Note: we&#39;re invoking the calls function here. Matcha.Trace . calls ( Integer , :parse , 2 ) # Will no longer be traced Integer . parse ( &quot;123&quot; ) Integer . parse ( &quot;123&quot; , 2 ) Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Recon tracer rate limit tripped . { 1 , &quot;23&quot; } But, what if we want to be even more specific about what base we are interested in?","ref":"guide-tracing.html#tracing-specific-arities"},{"type":"extras","title":"...for Tracing - Tracing Specific Arguments","doc":"Elixir also supports function overloading , providing different implementations for a function depending on the specific kinds of arguments it is invoked with. In many languages, this is acheived based on a type system; in Elixir, we use pattern matching in the function head and guards to steer our function calls to a specific implementation. We can use match specifications to complement this feature, and describe exactly what sorts of arguments we are interested in receiving trace messages for. For example, imagine that we are only trying to understand how often we parse a string into a binary number—even though a bizzare part of our codebase is also parsing things into trinary numbers as well. If we provide the tracing engine with a match specification using the Matcha.spec/2 macro, we can pattern match on precisely the arguments we want to be notified about: require Matcha spec = Matcha . spec :trace do [ _ , 2 ] -&gt; true end Matcha.Trace . calls ( Integer , :parse , spec ) # Will no longer be traced Integer . parse ( &quot;123&quot; , 3 ) Integer . parse ( &quot;123&quot; , 2 ) Matcha.Trace : ` Elixir.Integer . parse ( &quot;123&quot; , 2 ) ` called on # PID &lt; 0 . 142 . 0 &gt; Recon tracer rate limit tripped . Matcha.Trace : ` Elixir.Integer . parse ( &quot;123&quot; , 2 ) ` called on # PID &lt; 0 . 142 . 0 &gt; Recon tracer rate limit tripped . { 1 , &quot;23&quot; } Tracing match specifications always match on a single thing: the list of the arguments provided to our function calls. (What we do in the match body is elaborated on more below in the Tracing Functions section. For now, know that returning true is a sane default for arguments we are interested in.) Any time our function is called with arguments that match one of our specified patterns, we will receive a message. In the example above, we are describing that we are only interested in functions called with two parameters, where the first parameter can be anything, and the second parameter is the literal value 2 . Hence, we match on [_, 2] . Combined with the exact module and function name we are interested in tracing, we can be very precise in what we want to receive messages for! We can accomplish tracing on specific arguments even more tersely, without constructing a spec and passing it along with a temporary variable, with the Matcha.trace_calls/3 macro: require Matcha Matcha . trace_calls Integer , :parse do [ _ , 2 ] -&gt; true end # Will not be traced Integer . parse ( &quot;123&quot; , 3 ) Integer . parse ( &quot;123&quot; , 2 ) { 1 , &quot;23&quot; } Matcha.Trace : ` Elixir.Integer . parse ( &quot;123&quot; , 2 ) ` called on # PID &lt; 0 . 142 . 0 &gt; Recon tracer rate limit tripped . What else can we do when tracing specific arguments?","ref":"guide-tracing.html#tracing-specific-arguments"},{"type":"extras","title":"...for Tracing - Functions Available When Tracing","doc":"It turns out that the tracing engine supports a lot of customization! When tracing with match specifications, we can use special functions to send trace messages with additional context, provide a completely different format for our messages, and modify how the tracing engine will behave once certain function calls are matched. This is not a commonly-reached-for feature, and requires a deeper dive into Erlang documentation to learn how to fully utilize it. All available functions used in tracing are documented in Matcha.Context.Trace . A couple of things to note: Despite being declared in Matcha.Context.Trace , when used in tracing match specifications, you do not prefix the function call with the module name. For example, we would invoke the Matcha.Context.Trace.return_trace/0 function unadorned, like: Matcha . trace_calls ( Integer , :parse ) do [ _ , 2 ] -&gt; return_trace ( ) end These tracing functions cannot be used in just any match specification. The Matcha.trace_calls macro builds a match specification with the correct Matcha.Context.Trace ; when building specs by hand you must clarify that you are building a tracing one: # Will fail with a `(CompileError): undefined function return_trace/0`, # as the function does not exist outside of a tracing context. spec = Matcha . spec do [ _ , 2 ] -&gt; return_trace ( ) end # By clarifying that we intend to use this spec in a `:trace` context, # tracing functions are now available to us. spec = Matcha . spec :trace do [ _ , 2 ] -&gt; return_trace ( ) end Matcha.Trace . calls ( Integer , :parse , spec ) For more details on these special tracing-only functions, consult the Matcha.Context.Trace documentation (and linked Erlang documentation therein). As a somewhat contrived example: let's say that even when we have a large :limit , we want to supress tracing messages after a function call with specific arguments is traced: require Matcha Matcha . trace_calls Integer , :parse , limit : 1_000 do # Receive a message for all trinary calls [ _ , 3 ] -&gt; message ( &quot;integer parsed into trinary&quot; ) # But, stop receiving any matching calls once a binary one is detected [ _ , 2 ] -&gt; message ( &quot;integer parsed into binary&quot; ) trace ( [ ] , [ :silent ] ) end # Will generate a trace message Integer . parse ( &quot;123&quot; , 3 ) # as many times as we want Integer . parse ( &quot;123&quot; , 3 ) # But, once a binary call is detected, no further tracing will happen Integer . parse ( &quot;123&quot; , 2 ) # even when other matching functions are called Integer . parse ( &quot;123&quot; , 3 ) { 5 , &quot;3&quot; } Matcha.Trace : ` Elixir.Integer . parse ( &quot;123&quot; , 3 ) ` called on # PID &lt; 0 . 142 . 0 &gt; : integer parsed into trinary Matcha.Trace : ` Elixir.Integer . parse ( &quot;123&quot; , 3 ) ` called on # PID &lt; 0 . 142 . 0 &gt; : integer parsed into trinary Setting aside the structure of our function calls, there are other ways to filter what trace messages we recieve.","ref":"guide-tracing.html#functions-available-when-tracing"},{"type":"extras","title":"...for Tracing - Tracing Specific Processes","doc":"A very effective way of specifying exactly what you want to trace is by being more specific about where you want to receive trace messages from. You can limit what is traced to specific processes through a few different means. Tracing Individual Processes The most intuitive mechanism is to trace a single specific pid . This lets us not worry about all the noise generated by function calls across your entire distributed system! For example, we can only trace calls in our current process, and not worry about what other processes are calling: IO . puts ( &quot;I am: \#{ inspect ( self ( ) ) } &quot; ) # Configure the tracing engine to only monitor our `self()` Matcha.Trace . module ( Integer , limit : 1_000 , pid : self ( ) ) do_integer_things = fn -&gt; IO . puts ( &quot;The process now executing some Integer calls is: \#{ inspect ( self ( ) ) } &quot; ) Integer . digits ( 123 ) Integer . parse ( &quot;1&quot; ) end # Calls from this process will emit tracing messages do_integer_things . ( ) # The same calls in other processes will not be traced! IO . puts ( &quot;A newly spawned process is: \#{ inspect ( spawn ( do_integer_things ) ) } &quot; ) I am : # PID &lt; 0 . 142 . 0 &gt; The process now executing some Integer calls is : # PID &lt; 0 . 142 . 0 &gt; A newly spawned process is : # PID &lt; 0 . 328 . 0 &gt; The process now executing some Integer calls is : # PID &lt; 0 . 328 . 0 &gt; :ok Matcha.Trace : ` Elixir.Integer . digits ( 123 ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . digits ( 123 ) ` called on # PID &lt; 0 . 328 . 0 &gt; Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 328 . 0 &gt; Tracing Groups of Processes We can also use some special atoms to describe the processes we want to trace based on when they are created, relative to when tracing starts. For example, we might want to only trace :existing processes, and ensure that any new process that gets spawned while we are tracing neither emits a message, nor counts against our limit: IO . puts ( &quot;I am: \#{ inspect ( self ( ) ) } &quot; ) # Configure the tracing engine to only monitor `:existing` processes Matcha.Trace . module ( Integer , limit : 1_000 , pid : :existing ) do_integer_things = fn -&gt; IO . puts ( &quot;The process now executing some Integer calls is: \#{ inspect ( self ( ) ) } &quot; ) Integer . digits ( 123 ) Integer . parse ( &quot;1&quot; ) end # Calls from this process, which existed when we started tracking, will emit tracing messages. do_integer_things . ( ) # The same calls in processes created after we started tracing will not be traced! IO . puts ( &quot;A newly spawned process is: \#{ inspect ( spawn ( do_integer_things ) ) } &quot; ) I am : # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . __info__ ( :deprecated ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . __info__ ( :deprecated ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . __info__ ( :module ) ` called on # PID &lt; 0 . 142 . 0 &gt; The process now executing some Integer calls is : # PID &lt; 0 . 142 . 0 &gt; A newly spawned process is : # PID &lt; 0 . 334 . 0 &gt; The process now executing some Integer calls is : # PID &lt; 0 . 334 . 0 &gt; :ok Matcha.Trace : ` Elixir.Integer . digits ( 123 ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . digits ( 123 ) ` called on # PID &lt; 0 . 334 . 0 &gt; Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 334 . 0 &gt; Alteratively, we could only trace :new processes for the opposite effect: only processes that are spawned after we start tracing will send messages: IO . puts ( &quot;I am: \#{ inspect ( self ( ) ) } &quot; ) # Configure the tracing engine to only monitor `:new` processes Matcha.Trace . module ( Integer , limit : 1_000 , pid : :new ) do_integer_things = fn -&gt; IO . puts ( &quot;The process now executing some Integer calls is: \#{ inspect ( self ( ) ) } &quot; ) Integer . digits ( 123 ) Integer . parse ( &quot;1&quot; ) end # Calls from this process, which existed when we started tracking, not be traced. do_integer_things . ( ) # The same calls in new processes created after we started tracing will be! IO . puts ( &quot;A newly spawned process is: \#{ inspect ( spawn ( do_integer_things ) ) } &quot; ) Matcha.Trace : ` Elixir.Integer . __info__ ( :deprecated ) ` called on # PID &lt; 0 . 142 . 0 &gt; I am : # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . __info__ ( :deprecated ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . __info__ ( :module ) ` called on # PID &lt; 0 . 142 . 0 &gt; The process now executing some Integer calls is : # PID &lt; 0 . 142 . 0 &gt; A newly spawned process is : # PID &lt; 0 . 337 . 0 &gt; The process now executing some Integer calls is : # PID &lt; 0 . 337 . 0 &gt; :ok Matcha.Trace : ` Elixir.Integer . digits ( 123 ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 142 . 0 &gt; Matcha.Trace : ` Elixir.Integer . digits ( 123 ) ` called on # PID &lt; 0 . 337 . 0 &gt; Matcha.Trace : ` Elixir.Integer . parse ( &quot;1&quot; ) ` called on # PID &lt; 0 . 337 . 0 &gt; You can also specify the process group pid: :all ; however, this is the default behaviour so is not very useful in practice.","ref":"guide-tracing.html#tracing-specific-processes"},{"type":"extras","title":"Adopting Matcha","doc":"Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } , { :ex2ms , &quot;&gt;= 0.0.1&quot; } ] , force : true )","ref":"guide-adoption.html"},{"type":"extras","title":"Adopting Matcha - Overview","doc":"Matchspecs can be arcane, load-bearing spells at critical points in your codebase. If you are considering adopting Matcha , here are some tips to help you transition. This is an in-depth guide to adopting Matcha in your projects. For a quick reference, see the adoption cheatsheet . The first part of this guide is targeted at projects already using match specifications, that want to adopt Matcha to compose them. If you simply want to plug your existing specs into Matcha 's APIs, or only use Matcha for new match specs, you can skip to the &quot;Choosing Matcha Contexts&quot; section below.","ref":"guide-adoption.html#overview"},{"type":"extras","title":"Adopting Matcha - Reading Raw Specs","doc":"Initially, you'll want to read your original spec to try to understand what it is doing, and convert it mentally into Elixir code. This may require learning the entire Erlang match spec grammar . Matcha does not currently have a syntax guide to Erlang's match specs, but we intend to develop one , and will replace this callout with a link to it and a walkthrough when it is launched. If you are moving raw, handwritten specs to Matcha , you can skip to the &quot;Wrapping Raw Specs&quot; section below.","ref":"guide-adoption.html#reading-raw-specs"},{"type":"extras","title":"Adopting Matcha - Comparing Generated Specs","doc":"If you are using another Elixir-to-MS compiler, such as ex2ms , you already have Elixir code that describes what you are trying to do! Now, you must convince yourself that Matcha will do the same thing with it. The simplest way to get started with this is to compare both compilers' output. We'll start by requiring both compiler's macros: require Ex2ms require Matcha Now, let's take this example from the ex2ms documentation: ex2ms_spec = Ex2ms . fun do { x , y } = z when x &gt; 10 -&gt; z end To see what Matcha compiles this to, we simply replace calls to Ex2ms.fun/1 with Matcha.spec/1 : matcha_spec = Matcha . spec :table do { x , y } = z when x &gt; 10 -&gt; z end You'll notice a couple things running these examples. Firstly, the Matcha version emitted an Elixir compiler warning: variable &quot;y&quot; is unused ! This is a core feature of Matcha : the Elixir code you give it is passed through the Elixir compiler to ensure that all useful warnings about your match specification code are preserved and emitted as expected. When moving over to Matcha , you may find new opportunities to clean your match specification code up, and hold it to the same standard as your Elixir code. Secondly, we do not get back raw Erlang match specification source code; instead, our spec is wrapped in a Matcha.Spec struct . This lets it play nicely with high-level Matcha APIs. If instead we want to access the raw specification source code, for example to compare it to ex2ms output or pass it into an Erlang API, we can call Matcha.Spec.source/1 : Matcha.Spec . source ( matcha_spec ) At the time of writing, both of these compilers produce the same raw Erlang match spec for this example. This is not guaranteed, however—they well may produce semantically equivalent, syntactically different match specifications as the compilers evolve. If you want to see if they produce different results, ExUnit.Assertions.assert/1 can let us know if they are the same, or cleanly depict how they are different: require ExUnit.Assertions ExUnit.Assertions . assert ( Matcha.Spec . source ( matcha_spec ) == ex2ms_spec ) Via this mechanism, you can study the differences and similarities between the compilers and satisfy yourself that both tools produce similar, if not identical, match specifications. If you want to convince yourself that these do the same thing, see &quot;Reading Raw Specs&quot; above. If you want the computer to convice you, continue reading &quot;Testing Specs With Erlang APIs&quot; below. SUMMARY You can move from ex2ms to Matcha by changing Ex2ms.fun do... to Matcha.spec do... Matcha.spec produces a Matcha.Spec struct, with an Erlang-API-ready spec from Matcha.Spec.source/1 Both compilers may produce different but semantically equivalent specs Matcha will issue familiar Elixir compiler warnings for the Elixir code you give it","ref":"guide-adoption.html#comparing-generated-specs"},{"type":"extras","title":"Adopting Matcha - Wrapping Raw Specs","doc":"If you are not using the Macro.spec/1 macro to build your specs, you can still wrap existing ones for usage with Matcha APIs: simply provide them to Matcha.Spec.from_source!/1 to get a Matcha.Spec struct. For example: raw_spec = [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&gt; , :&quot;$1&quot; , 10 } ] , [ :&quot;$_&quot; ] } ] { :ok , matcha_spec } = Matcha.Spec . from_source! ( raw_spec )","ref":"guide-adoption.html#wrapping-raw-specs"},{"type":"extras","title":"Adopting Matcha - Testing Specs With Erlang APIs","doc":"","ref":"guide-adoption.html#testing-specs-with-erlang-apis"},{"type":"extras","title":"Adopting Matcha - Choosing Matcha Contexts","doc":"At this point, you have Matcha.Spec structs you want to use, either from: Using the Matcha.spec/1 macro to build them Using Matcha.Spec.from_source!/1 to wrap existing ones We can access their raw match spec source with Matcha.Spec.source/1 and pass them to Erlang APIs. If we want to use Matcha APIs instead, however, we will want to provide them with a context , which both of the above functions accept as an optional parameter. Erlang match specs aren't just a DSL with their own grammar, they actually describe several different grammars depending on how you intend to use the spec. Matcha encodes this intention as a Matcha.Context , and handles them differently at both compile-time and runtime to enforce different guarantees and support different use-cases. The Matcha.Context documentation goes into this in more depth, but for all practical purposes the quick rundown of them is: Use the (default) :filter_map or :match contexts if you intend to play with specs and in-memory data, using the Matcha.Spec call/2 , run/2 , and stream/2 functions Use the :trace context if you intend to query data with Matcha.Trace functions Use the :table context if you intend to trace code execution with the Matcha.Table functions","ref":"guide-adoption.html#choosing-matcha-contexts"},{"type":"extras","title":"Adopting Matcha - Using Matcha APIs","doc":"Now that you're building Matcha specs with the correct context for your use-case, you're ready to check out the usage guides and explore Matcha APIs!","ref":"guide-adoption.html#using-matcha-apis"},{"type":"extras","title":"Adoption Cheatsheet","doc":"Convert your project to Matcha quickly! For a more in-depth description of how to move your project over, see the adoption guide . At a high level, you will want to: Choose a Matcha.Context for each spec Build a Matcha.Spec in that context Use that spec in the corresponding Matcha APIs","ref":"cheatsheet-adoption.html"},{"type":"extras","title":"Adoption Cheatsheet - Choosing a Matcha.Context","doc":"The :table context Use the :table context if you intend to trace code execution with the Matcha.Table functions. require Matcha matcha_spec = Matcha . spec ( :table ) do { x , y } = z when x &gt; 10 -&gt; z end The :trace context Use the :trace context if you intend to query data with Matcha.Trace functions. require Matcha matcha_spec = Matcha . spec ( :table ) do { x , y } = z when x &gt; 10 -&gt; z end Other contexts Use the :filter_map or :match contexts if you intend to play with specs and in-memory data using the Matcha.Spec . require Matcha matcha_spec = Matcha . spec ( :table ) do { x , y } = z when x &gt; 10 -&gt; z end","ref":"cheatsheet-adoption.html#choosing-a-matcha-context"},{"type":"extras","title":"Adoption Cheatsheet - Creating Matcha.Spec structs","doc":"Building Specs With Elixir Matcha provides an Elixir-to-Matcha compiler with the Matcha.spec/2 macro. require Matcha matcha_spec = Matcha . spec ( :table ) do { x , y } = z when x &gt; 10 -&gt; z end Wrapping Raw Specs You can make existing raw match specs play nice with Matcha APIs using Matcha.Spec.from_source!/2 raw_spec = [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&gt; , :&quot;$1&quot; , 10 } ] , [ :&quot;$_&quot; ] } ] matcha_spec = Matcha.Spec . from_source! ( :table , raw_spec )","ref":"cheatsheet-adoption.html#creating-matcha-spec-structs"},{"type":"extras","title":"Adoption Cheatsheet - Using Matcha Specs","doc":"Outside of Matcha You can always extract the raw source of any Matcha.Spec and pass it into other APIs that do not support Matcha with Matcha.Spec.source/1 . require Matcha matcha_spec = Matcha . spec ( :table ) do { x , y } = z when x &gt; 10 -&gt; z end raw_spec = Matcha.Spec . source ( matcha_spec ) call/2 , run/2 , and stream/2 functions.","ref":"cheatsheet-adoption.html#using-matcha-specs"},{"type":"extras","title":"Tracing Cheatsheet","doc":"Get started with tracing quickly! For a more in-depth introduction to tracing, see the tracing guide .","ref":"cheatsheet-tracing.html"},{"type":"extras","title":"Tracing Cheatsheet - Limiting Tracing","doc":"Since tracing can send a lot of messages to the tracing process, several guard rails are in place by default. However, these limits can be configured. Limiting Traced Messages By default, all traces will be limited to a single message: Matcha.Trace . module ( Integer ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; #=&gt; Recon tracer rate limit tripped. Integer . parse ( &quot;2&quot; ) Extend this with the :limit option: Matcha.Trace . module ( Integer , limit : 2 ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; Integer . parse ( &quot;2&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; #=&gt; Recon tracer rate limit tripped. Limiting Traced Processes By default, calls in all processes will be traced: Matcha.Trace . module ( Integer , limit : 2 ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; spawn ( fn -&gt; Integer . parse ( &quot;1&quot; ) end ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.320.0&gt; Specify which processes to trace with the :pid option: Matcha.Trace . module ( Integer , pid : :all , limit : 2 ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; spawn ( fn -&gt; Integer . parse ( &quot;1&quot; ) end ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.320.0&gt; Limit tracing to a specific process or list of processes: Matcha.Trace . module ( Integer , pid : self ( ) , limit : 2 ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; spawn ( fn -&gt; Integer . parse ( &quot;1&quot; ) end ) Limit tracing to just existing processes: Matcha.Trace . module ( Integer , pid : :existing , limit : 2 ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; spawn ( fn -&gt; Integer . parse ( &quot;1&quot; ) end ) Limit tracing to just newly spawned processes: Matcha.Trace . module ( Integer , pid : :new , limit : 2 ) Integer . parse ( &quot;1&quot; ) spawn ( fn -&gt; Integer . parse ( &quot;1&quot; ) end ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.320.0&gt;","ref":"cheatsheet-tracing.html#limiting-tracing"},{"type":"extras","title":"Tracing Cheatsheet - Tracing Calls","doc":"Tracing Calls to Entire Modules Use Matcha.Trace.module/1 : Matcha.Trace . module ( Integer ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; Tracing Calls to Specific Functions Use Matcha.Trace.function/2 : Matcha.Trace . function ( Integer , :parse ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; Tracing Calls with Specific Arities Use Matcha.Trace.calls/3 with a numeric arity: Matcha.Trace . calls ( Integer , :parse , 1 ) Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt; Tracing Calls with Match Specs Use the Matcha.trace_calls/3 macro: require Matcha Matcha . trace_calls ( Integer , :parse , limit : 2 ) do [ &quot;1&quot; ] -&gt; message ( &quot;Parsing ` \\&quot; 1 \\&quot; ` into a decimal number&quot; ) [ &quot;1&quot; , 2 ] -&gt; message ( &quot;Parsing ` \\&quot; 1 \\&quot; ` into a binary number&quot; ) end Integer . parse ( &quot;1&quot; ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt;: Parsing `&quot;1&quot;` into a decimal number Integer . parse ( &quot;1&quot; , 2 ) #=&gt; Matcha.Trace: `Elixir.Integer.parse(&quot;1&quot;)` called on #PID&lt;0.286.0&gt;: Parsing `&quot;1&quot;` into a binary number","ref":"cheatsheet-tracing.html#tracing-calls"},{"type":"extras","title":"Changelog","doc":"Matcha uses Semantic Versioning 2.0.0 . Changes before v0.2.0 are considered pre-release and not included here.","ref":"changelog.html"},{"type":"extras","title":"Contributing","doc":"Thanks for considering contributing to Matcha!","ref":"contributing.html"},{"type":"extras","title":"Contributing - Quickstart","doc":"Assuming you have the GitHub cli and the asdf version manager installed: # Acquire a copy of the code gh repo fork git@github.com:christhekeele/matcha.git --remote cd matcha # Install dependencies asdf install mix deps.get # Start work git checkout -b &lt;some-branch-name&gt; # ...do changes, commits, pushes, etc mix checks # ...address reported test failures, type issues, linting problems # Submit PR gh pr create If you are not using gh or asdf , you should fork Matcha from the source and set up your Erlang and Elixir installations on your machine to match those described in .tool-versions yourself.","ref":"contributing.html#quickstart"},{"type":"extras","title":"Contributing - What to Contribute","doc":"We cultivate a set of issues that are good ways to contribute to Matcha for the first time. Looking for other ways to contribute? Consider: Submitting a PR See a typo or a broken link? Is existing documentation unclear, or does following it lead to behaviour you consider surprising? Jump in and help us correct it! Helping answer questions in the communituy Helping other people use Matcha successfully is a great way to give back! Improving the documentation Documentation is the most valuable contribution you can make directly to the repository! It is easy to overlook documentation when programming, and difficult to look back on something you understand and see where others may get confused. This makes it hard to keep documentation high-quality, and all assistance in fighting entropy is invaluable! Tackling something that doesn't require extensive knowledge of the codebase These are less-involved issues that should be approachable without spending a bunch of time studying the entire project. They generally touch parts of the library that are similar across other Elixir and open-source projects. Addressing regressions in upcoming releases to the language Matcha continuously looks ahead to upcoming language releases, running its test suite against them to anticipate compatibility issues. If a storm is brewing on the horizon, the bleeding-edge test suite normally catches it, with more than enough time for someone to jump in and deal with the regression. Starting a discussion Have an idea for an improvement or an enhancement, but don't see a issue for it yet? Crack open a discussion and flesh it out with the maintainers! Opening a new issue Is something else the matter, or do the ideas above not fit what you have in mind? Create an issue and continue the conversation!","ref":"contributing.html#what-to-contribute"},{"type":"extras","title":"Contributing - Code","doc":"Want to contribute code? Here's what you need to know. Project Structure Matcha is a pretty standard Elixir library, and should be navigable to anyone familiar with such things. Here is the map, with points of interest where it may deviate from a typical project: matcha / │ ├ ─ ─ CONTRIBUTING . md # YOU ARE HERE ├ ─ ─ README . md # Project landingpad ├ ─ ─ CHANGELOG . md # Describes changes in each release ├ ─ ─ LICENSE . md # License Matcha is available under │ ├ ─ ─ mix . exs # Project manifest │ ├ ─ ─ VERSION # Library version ├ ─ ─ lib / # Library source code │ ├ ─ ─ test / # Test suite │ ├ ─ ─ benchmark / # Benchmark suite │ ├ ─ ─ doctest / # Tests of moduledocs as laid out in lib/ │ ├ ─ ─ unit / # Tests modules as laid out in lib/ │ └ ─ ─ usecases / # Tests derived from realworld ms usage │ ├ ─ ─ docs / # Extra material for docgen │ ├ ─ ─ guides / # Interactive livebook guides │ └ ─ ─ img / # Images used in docgen │ ├ ─ ─ benchmarks / # Benchmark reports generated by `mix test.benchmarks` ├ ─ ─ coverage / # Coverage reports generated by `mix test.coverage` └ ─ ─ doc / # Documentation generated by `mix docs` Branches &amp; Tags latest is the default integration branch where work comes together. This means that you can get the &quot;cutting edge&quot; version of Matcha via: Mix . install matcha : [ github : &quot;christhekeele/matcha&quot; , ref : &quot;latest&quot; ] , force : true release is the staging branch where code intended for the next release is placed. This means that you can get the &quot;release candidate&quot; version of Matcha via: Mix . install matcha : [ github : &quot;christhekeele/matcha&quot; , ref : &quot;release&quot; ] , force : true stable is a floating tag pointing to the &quot;highest&quot; semantic version of Matcha released to hex.pm . This means that the &quot;latest official&quot; version of Matcha is available identically via: Mix . install matcha : [ github : &quot;christhekeele/matcha&quot; , ref : &quot;stable&quot; ] , force : true and Mix . install matcha : &quot;&gt;= 0.0.0&quot; , force : true tags starting with v , ex vX.Y.Z-mayberc , represent semantic versions published to hex.pm . If versions must be modified or yanked, currently these tags should be deleted or moved manually. This means that these two are equivalent: Mix . install matcha : [ github : &quot;christhekeele/matcha&quot; , ref : &quot;vX.Y.Z-mayberc&quot; ] , force : true and Mix . install matcha : &quot;vX.Y.Z-mayberc&quot; , force : true All other branch or tag names are fair game.","ref":"contributing.html#code"},{"type":"extras","title":"Contributing - Docs","doc":"Code Documentation Matcha uses ex_doc to generate documentation from source code automatically on every release. If you're developing documentation, you can preview it locally by running: mix docs open doc/index.html The warnings emitted by ex_doc are useful and worth keeping an eye on! You can also get and overview of the current documentation coverage status by running: mix docs.coverage Guides Matcha also includes a set of interactive guides powered by LiveBook . They are maintained in the docs/guides folder. They are best developed against a local instance of livebook, via: LIVEBOOK_TOKEN_ENABLED=false livebook server --root-path docs/guides Avoid having the .livemd files also open in an editor as you work on a guide, to avoid getting into save-tug-of-war!","ref":"contributing.html#docs"},{"type":"extras","title":"Contributing - Checks","doc":"Matcha has three different checks that may run during various automatic builds. If you want to get ahead of build failures, you can run them all locally before pushing up code with the command mix checks . This is equivalent to running: mix test Runs the tests found in test/ , checking for failures. mix typecheck Runs dialyzer , checking for provable type issues. mix lint Checks for compiler warnings, formatting divergences, and style problems . CI Suites Matcha has 5 test suites that run different checks automatically, depending on what's happening, for different versions of Erlang/OTP, Elixir, and Matcha's dependencies. Versions The sets of versions we run checks against are named: preferred otp : The latest minor version of the highest major version Erlang we want to support elixir : The latest patch version of highest minor Elixir we want to support deps : The locked-down version of our dependencies in our mix.lock matrix otp : The latest minor versions of every major Erlang version we want to support elixir : The latest patch versions of every minor Elixir version we want to support deps : The locked-down version of our dependencies in our mix.lock edge otp : The latest major version Erlang elixir : The upcoming version of Elixir available on its default branch deps : The un-locked version of dependencies in our mix.exs Workflows The automated test workflows we run are: Test Suite Runs on every set of commits pushed up to GitHub, on source or forked repositories. Runs all mix checks for the preferred versions of our dependencies. Provides continuous feedback on every potential change to the codebase. Test Status Runs on every set of commits and every PR into the latest and release branches. Runs the Test Suite , and updates related code quality services about its robustness. Provides insight into test meta-data like code coverage, documentation quality, etc. displayed in the README.md. Test Matrix Runs on every set of commits and every PR into the latest and release branches. Runs all mix checks for every set of versions in our matrix of dependencies. Provides full feedback on if each approved change will work on all supported versions. Test Release Runs on every set of commits added to pull requests to the release branch. Performs a dry-run of a planned release. Provides a preview of what a release would look like if published from the release branch. Test Edge Runs automatically UTC midnight. Runs all mix checks for the edge versions of our dependencies. Provides continuous feedback on how prepared the codebase is for upstream changes in dependencies.","ref":"contributing.html#checks"},{"type":"extras","title":"Contributors","doc":"🍵 Thanks to the good folk who've spent time to make Matcha what it is! 🍵 We've tried to mention everyone who has directly contributed inspiration, ideas, feedback, bug reports, discussion, or commits that ultimately impact the codebase. Of course, indirectly, this applies to the entire BEAM VM community, but we have only so many bytes to fit in these narrow margins. Thank you all, as well! @axelson Contributor @christhekeele Maintainer @ericmj Inspiration @hissssst Contributor @jhogberg Enabler @josevalim Inspiration @mguilmineau Contributor @OnorioCatenacci Contributor @tmr08c Contributor @TwistingTwists Contributor","ref":"contributors.html"},{"type":"extras","title":"Contributors - Notes","doc":"Handles, profile pictures, and links are taken from first point of public contribution. No other information is contained in these attributions. Attributions in alphabetical order by handle. Corrections, additions, and removals welcome via PR.","ref":"contributors.html#notes"},{"type":"extras","title":"License","doc":"Copyright © 2020 Christopher Keele Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}]